/**@file
 *@brief	
 *@date		Sat,31 Aug,2013 - Tue,01 Oct,2013
 *@author	Copyright(C)2001-2013 G-HAL, All rights reserved.
 *
 *@note
 *@verbatim
Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions
are met:

* Redistributions of source code must retain the above copyright
notice, this list of conditions and the following disclaimer.

* Redistributions in binary form must reproduce the above copyright
notice, this list of conditions and the following disclaimer in the
documentation and/or other materials provided with the distribution.

* Neither the name of the copyright holders nor the names of its contributors
may be used to endorse or promote products derived from this software
without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
"AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE COPYRIGHT OWNER OR
CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *@endverbatim
 */

#if defined(HAVE_CONFIG_H)
# include "config.h"
#endif

#if defined(HAVE_STDLIB_H)
# include <stdlib.h>
#endif
#if defined(HAVE_STDIO_H)
# include <stdio.h>
#endif
#if defined(HAVE_STDBOOL_H)
# include <stdbool.h>
#endif

#if defined(HAVE_ASSERT_H)
# include <assert.h>
#endif

#if defined(HAVE_ERRNO_H)
# include <errno.h>
#endif
#if defined(HAVE_UNISTD_H)
# include <unistd.h>
#endif
#if defined(HAVE_STRING_H)
# include <string.h>
#endif
#if defined(HAVE_SYS_SOCKET_H)
# include <sys/socket.h>
#endif
#if defined(HAVE_SYS_TYPES_H)
# include <sys/types.h>
#endif
#if defined(HAVE_PWD_H)
# include <pwd.h>
#endif

#if defined(HAVE_BSD_GETOPT_H)
# include <bsd/getopt.h>
#endif
#if defined(HAVE_GETOPT_LONG)
# include <getopt.h>
#endif

#if defined(HAVE_LANGINFO_H)
# include <langinfo.h>
#endif

#include "intl.h"
#include "check_kanji_encoding.h"
#include "alternative_strlcpy.h"
#include "logger.h"
#include "args.h"



struct args_t args;



#if defined(HAVE_GETOPT_LONG)
#else
typedef enum{
	no_argument, required_argument, optional_argument
} HasArg;

struct option{
	const char* name;
	HasArg has_arg;
	int* flag;
	int val;
};


/** libgnugetopt شؿ
 *@param			argc				ο
 *@param[in,out]	argv				ϡϡʸѴΰ
 *@param[in]		long_options		ĹȣʸΰѴơ֥
 *
 *@author	G-HAL
 *@date		Sat,31 Aug,2013 - Sun,01 Sep,2013
 */
static void getopt_long_translate(int argc, char* const argv[], const struct option* const long_options)
{
	assert (NULL != argv);
	assert (NULL != long_options);

	for (int i = 0; i < argc; ++i) {
		if ('-' == argv[i][0]) {
		  if ('-' == argv[i][1]) {
			bool hit = false;
			for (size_t j = 0; '\0' != long_options[j].name; ++j) {
				if (0 == strcmp(&(argv[i][2]), long_options[j].name)) {
					hit = true;
					if (NULL != long_options[j].flag) {
						(*(long_options[j].flag)) = long_options[j].val;
					}
					argv[i][1] = long_options[j].val;
					argv[i][2] = '\0';
					break;
				}
			}
			if (!hit) {
				argv[i][1] = '?';
			}
		  }
		}
	}
	return;
}
#endif


/** version ɽ
 *
 *@author	G-HAL
 *@date		Sat,31 Aug,2013 - Sun,01 Sep,2013
 */
static void print_version(void)
{
	fprintf(stdout,"%s: - %s\n", PACKAGE_NAME, PACKAGE_VERSION);

	fprintf(stdout,_(N_("\tCompile Option for Debugging: ")));
  #if defined(DEBUG)
	fprintf(stdout,"DEBUG=%d ", (int)(DEBUG));
  #endif
  #if defined(SIGNAL_UNSAFE)
	fprintf(stdout,"SIGNAL_UNSAFE ");
  #endif
	fprintf(stdout,_(N_("\n")));

	return;
}

/** credits ɽ
 *
 *@author	G-HAL
 *@date		Sat,31 Aug,2013 - Sun,01 Sep,2013
 */
static void print_credits(void)
{
	fprintf(stdout,_(N_("\tBSD License\tCopyright(C)2013 G-HAL, All rights reserved.\n")));
	fprintf(stdout,_(N_("\n")));
	return;
}

/** usage ɽ
 *
 *@author	G-HAL
 *@date		Sat,31 Aug,2013 - Sun,01 Sep,2013
 *@date		Mon,30 Sep,2013
 */
static void print_useage(void)
{
	fprintf(stdout,_(N_("useage: %1$s [option(s)]...\n")), args.env.name);
	fprintf(stdout,_(N_("\n")));
	fprintf(stdout,
			_(N_(" Options are:\n"
				 "      --unix\t\t\t\tPF_UNIX mode.\n"
				 "      --inet\t\t\t\tPF_INET mode.\n"
				 "      --socket-name (path to socket)\tName of socket (only PF_UNIX mode).\n"
				 "      --server-name (host name)\t\tName of server (only PF_INET mode).\n"
				 "      --service-name (port name)\tName of service (only PF_INET mode).\n"
				 "      --anonymous\t\t\tDo not learn.\n"
				 "      --string16-as-eucjp\t\tUse eucJP in a canna protocol. (default)\n"
				 "      --string16-as-utf16be\t\tUse UTF-16BE in a canna protocol. (NON-STANDARD)\n"
				 "  -H, --help\t\t\t\tShow these messages.\n"
				 "  -C, --credits\t\t\t\tShow credits.\n"
				 "      --version\t\t\t\tShow release version.\n")));
	fprintf(stdout,_(N_("\n")));
	return;
}



/** ꤵ줿Ŭ
 *@retval			true		
 *@retval			false		۾
 *
 *@author	G-HAL
 *@date		Sat,31 Aug,2013 - Sun,01 Sep,2013
 *@date		Mon,30 Sep,2013
 */
static bool check_args(void)
{
	bool	ret = true;

	if (args.cannaprotocol.unicode) {
		args.cannaprotocol.codeset = ICONV_UTF16BE;
	} else {
		args.cannaprotocol.codeset = ICONV_EUCJP;
	}
	{
		size_t len_str = strlen(args.cannaprotocol.area_lang) + 1 + strlen(args.cannaprotocol.codeset) + 1;
		char* const str = malloc(len_str);
		if (NULL == str) {
			ret = false;
			LOGGER_CRIT(l,N_("Out of memory at %1$s"), "check_args()");
			abort();
		}
		strlcpy(str, args.cannaprotocol.area_lang, len_str);
		strlcat(str, ".", len_str);
		strlcat(str, args.cannaprotocol.codeset, len_str);
		if (NULL != args.cannaprotocol.locale) {
			free(args.cannaprotocol.locale);
		}
		args.cannaprotocol.locale = str;
	}

	if (args.cannaprotocol.unicode) {
		args.codeset.internal = ICONV_UTF8;
	} else {
		args.codeset.internal = ICONV_EUCJP;
	}

	switch (args.server.domain) {
	case PF_UNIX:
	case PF_INET:
		break;
	default:
		LOGGER_WARN(l,N_("Illigal connection mode to a server."));
		ret = false;
		break;
	}

	return ret;
}


/** β
 *@param			argc		argv ͭĿ
 *@param[in,out]	argv		μΡgetopt_long̵ϡĹΥץ󤬣ʸηѴ
 *@retval			true		
 *@retval			false		۾
 *
 *@author	G-HAL
 *@date		Sat,31 Aug,2013 - Tue,03 Sep,2013
 *@date		Mon,30 Sep,2013
 */
bool parse_args(int argc, char* const* const argv)
{
	assert (NULL != argv);

	{
		char buf[STR_BUFSIZE] = "\0";
		for (int i = 0; i < argc; ++i) {
			strlcat(buf, argv[i], number_(buf));
			strlcat(buf, " ", number_(buf));
		}
		LOGGER_TRACE0(l,"argments; \"%s\"", buf);
	}

	static const struct option long_options[] = {
		{"unix",				no_argument,				NULL, 0xA0},
		{"inet",				no_argument,				NULL, 0xA1},
		{"socket-name",			required_argument,			NULL, 0xA4},
		{"server-name",			required_argument,			NULL, 0xA5},
		{"service-name",		required_argument,			NULL, 0xA6},
		{"anonymous",			no_argument,				NULL, 0xB0},
		{"string16-as-eucjp",	no_argument,				NULL, 0xC0},
		{"string16-as-utf16be",	no_argument,				NULL, 0xC1},
		{"help",				no_argument,	NULL, 'H'},
		{"version",				no_argument,	NULL, 0x80},
		{"credits",				no_argument,	NULL, 'C'},
		{0,no_argument,0,0}
	};
	static const char short_options[] = "*"
			"\xA0" "\xA1" "\xA4"":" "\xA5"":" "\xA6"":"
			"\xB0"
			"\xC0" "\xC1"
			"H" "\x80" "C";
	char argv01 = argv[0][1];
	int opt;
	_unused_ int longidx = 0;
	bool err = false, nocmd = false, someerr = false;


	if (argv01 != '*') {
		strlcpy(args.env.name, argv[0], number_(args.env.name));
	}

  #if defined(DEBUG)
	for (int i = 0; i < argc; ++i) {
		LOGGER_TRACE3(l,"%s", argv[i]);
	}
  #endif
  #if defined(HAVE_OPTRESET)
	optreset = 1;		/* for Standard C Library: IEEE Std 1003.2 (POSIX.2): getopt() */
  #endif
  #ifdef HAVE_GETOPT_LONG
	optind = 0;			/* for gnugetopt: getopt_long() */
  #else
	optind = 1;			/* for getopt: getopt() */
	getopt_long_translate( argc, argv, long_options );
  #endif
  #if defined(DEBUG)
	for (int i = 0; i < argc; ++i) {
		LOGGER_TRACE3(l,"%s", argv[i]);
	}
  #endif
  #ifdef HAVE_GETOPT_LONG
	while (-1 != (opt = getopt_long(argc, argv, short_options, long_options, &longidx)))
  #else
	while (-1 != (opt = getopt(argc, argv, short_options)))
  #endif
	{
		opt &= 0xFF;
		LOGGER_TRACE3(l,"%X,%X,%X", opt, longidx);
		switch (opt) {
		case '*':
			break;
		case ':':
			err = true;
			LOGGER_EMERG(l,N_("BUG!! short_options is broken."));
			break;
		case '?':
			err = true;
			LOGGER_ERR(l,N_("Unknown option(s)."));
			break;

		case 0xA0:
			args.server.domain = PF_UNIX;
			break;
		case 0xA1:
			args.server.domain = PF_INET;
			break;
		case 0xA4:
			strlcpy(args.server.socket_name, optarg, number_(args.server.socket_name));
			break;
		case 0xA5:
			strlcpy(args.server.server_name, optarg, number_(args.server.server_name));
			break;
		case 0xA6:
			strlcpy(args.server.service_name, optarg, number_(args.server.service_name));
			break;

		case 0xB0:
			args.user.anonymous = true;
			break;

		case 0xC0:
			args.cannaprotocol.unicode = false;
			break;
		case 0xC1:
			args.cannaprotocol.unicode = true;
			break;

		case 'H':
			print_version();
			print_credits();
			print_useage();
			if ('*' != argv01) {
				exit(0);
			}
			break;
		case 0x80:
			print_version();
			if ('*' != argv01) {
				exit(0);
			}
			break;
		case 'C':
			print_credits();
			if ('*' != argv01) {
				exit(0);
			}
			break;

		default:
			/* 顼å getopt() Ф */
			LOGGER_ERR(l,N_("Illigal option(s). %1$02X"), opt);
			err = true; nocmd = true;
			break;
		}
		if (err && !nocmd) {
			LOGGER_ERR(l,N_("Argment parameter error at %1$02X:'%2$s'."), opt, optarg);
		}
		someerr = (someerr || err);
		nocmd = false;
		err = false;
	}
	if (someerr || !check_args()) {
		LOGGER_ERR(l,N_("Argment parse error, or argment parameter error.\n"
						  "Try `%1$s --help' for more information"),
				   args.env.name);
	}

	return !err;
}



/** ΰɤ߹
 *@retval			true		
 *@retval			false		۾
 *
 *@author	G-HAL
 *@date		Sat,31 Aug,2013 - Tue,03 Sep,2013
 *@date		Tue,10 Sep,2013 - Wed,11 Sep,2013
 *@date		Mon,16 Sep,2013
 *@date		Sat,21 Sep,2013 - Sun,22 Sep,2013
 *@date		Mon,30 Sep,2013 - Tue,01 Oct,2013
 */
bool init_args(void)
{
	/* ѥ᡼ */
	strlcpy(args.env.name, "", number_(args.env.name));

	args.user.anonymous = false;
	const char* const user = getenv("USER");
	if ((NULL == user) || ('\0' == user[0])) {
		LOGGER_DEBUG(l,N_("Can not getenv(\"%1$s\"). %2$s"), "USER", strerror(errno));
		const uid_t uid = getuid();
		const struct passwd* pass = getpwuid(uid);
		if (NULL == pass) {
			LOGGER_WARN(l,N_("Can not getpwuid(). %1$s"), strerror(errno));
			strlcpy(args.user.name, "nobody", number_(args.user.name));
		} else {
			strlcpy(args.user.name, pass->pw_name, number_(args.user.name));
		}
	} else {
		strlcpy(args.user.name, user, number_(args.user.name));
	}
	const char* const group = getenv("GROUP");
	if (NULL == group) {
		LOGGER_INFO(l,N_("Can not getenv(\"%1$s\"). %2$s"), "GROUP", strerror(errno));
		strlcpy(args.user.group, "nobody", number_(args.user.group));
	} else {
		strlcpy(args.user.group, group, number_(args.user.group));
	}

	args.cannaprotocol.unicode = false;
	{
		const char* const env_lc_all = getenv("LC_ALL");
		const char* const env_lang = getenv("LANG");
		const char* const env_locale = (NULL != env_lc_all) ? env_lc_all : env_lang;
		if (NULL == env_locale) {
			args.cannaprotocol.area_lang = strdup("ja_JP");
		} else {
			char* const area_lang = strdup(env_locale);
			char* const ptr = strchr(area_lang, '.');
			if (NULL != ptr) {
				(*ptr) = '\0';
			}
			args.cannaprotocol.area_lang = area_lang;
		}
	}
	args.cannaprotocol.codeset = NULL; /* check_args() Ǽ롣 */
	args.cannaprotocol.locale = NULL; /* check_args() Ǽ롣 */

	args.codeset.sourcecode = SRC_KANJI_ENCODING;
	args.codeset.locale = nl_langinfo(CODESET);
	if (NULL == args.codeset.locale) {
		LOGGER_ERR(l,N_("Can not get nl_langinfo()."));
		args.codeset.locale = "";
	}
	args.codeset.internal = NULL; /* check_args() Ǽ롣 */

	strlcpy(args.server.socket_name, DEFAULT_SOCKET_NAME, number_(args.server.socket_name));
	strlcat(args.server.socket_name, ".", number_(args.server.socket_name));
	strlcat(args.server.socket_name, args.user.name, number_(args.server.socket_name));
	strlcpy(args.server.socket_name_fallback, DEFAULT_SOCKET_NAME, number_(args.server.socket_name_fallback));
	args.server.domain = PF_UNIX;
	strlcpy(args.server.server_name, DEFAULT_SOCKET_SERV, number_(args.server.server_name));
	strlcpy(args.server.service_name, DEFAULT_SOCKET_PORT, number_(args.server.service_name));


	if (!check_args()) {
		LOGGER_EMERG(l,N_("BUG!!\tAt initial data in args.c"));
		return false;
	}

	/* ǥեȰβ */
	{
		const char* const env_flg = getenv(CANNA_AGENT_FLAGS);
		if (NULL != env_flg) {
			char* const flg = strdup(env_flg);
			if (!flg) {
				LOGGER_CRIT(l,N_("Out of memory at %1$s"), "init_args()");
				abort();
			}
			int argc = 3;
			for (char* ptr = flg; '\0' != *ptr; ++ptr) {
				switch (*ptr) {
				case ' ':
				case '\t':
					(*ptr) = ' ';
					++argc;
					break;
				default:
					break;
				}
			}
			char** argv = malloc(sizeof(char*) * (unsigned int)argc);
			if (!argv) {
				LOGGER_CRIT(l,N_("Out of memory at %1$s"), "init_args()");
				abort();
			}
			char** ap = argv;
			char argv0[] = "-*";
			(*ap ++) = argv0; /* not real argv flag */
			(*ap ++) = flg;
			for (char* ptr = flg; '\0' != (*ptr); ++ptr) {
				if (' ' == (*ptr)) {
					(*ptr ++) = '\0';
					(*ap ++) = ptr;
					(*ap) = NULL;
				}
			}

			parse_args((argc - 1), argv);

			free(argv);
			free(flg);
		}
	}

	return true;
}



/** θ
 *
 *@author	G-HAL
 *@date		Sat,31 Aug,2013 - Sun,01 Sep,2013
 *@date		Mon,30 Sep,2013
 */
void fin_args(void)
{
	free(args.cannaprotocol.area_lang);
	free(args.cannaprotocol.locale);
	return;
}

/* [ End of File ] */
