/**@file
 *@brief	cannaץȥ򤷤٤ؿ
 *@date		Fri,23 Aug,2013 - Mon,30 Sep,2013
 *@date		Sat,04 Feb,2017
 *@author	Copyright(C)2013-2017 G-HAL, All rights reserved.
 *
 *@note
 *@verbatim
Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions
are met:

* Redistributions of source code must retain the above copyright
notice, this list of conditions and the following disclaimer.

* Redistributions in binary form must reproduce the above copyright
notice, this list of conditions and the following disclaimer in the
documentation and/or other materials provided with the distribution.

* Neither the name of the copyright holders nor the names of its contributors
may be used to endorse or promote products derived from this software
without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
"AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE COPYRIGHT OWNER OR
CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *@endverbatim
 */

#if defined(HAVE_CONFIG_H)
# include "config.h"
#endif

#if defined(HAVE_STDLIB_H)
# include <stdlib.h>
#endif
#if defined(HAVE_STDIO_H)
# include <stdio.h>
#endif
#if defined(HAVE_STDBOOL_H)
# include <stdbool.h>
#endif

#if defined(HAVE_ASSERT_H)
# include <assert.h>
#endif

#if !defined(__STDC_LIMIT_MACROS)
# define __STDC_LIMIT_MACROS
#endif
#if !defined(__STDC_CONSTANT_MACROS)
# define __STDC_CONSTANT_MACROS
#endif
#if defined(HAVE_STDINT_H)
# include <stdint.h>
#endif
#if !defined(__STDC_FORMAT_MACROS)
# define __STDC_FORMAT_MACROS
#endif
#if defined(HAVE_INTTYPES_H)
# include <inttypes.h>
#endif

#if defined(HAVE_STRING_H)
# include <string.h>
#endif
#if defined(HAVE_ERRNO_H)
# include <errno.h>
#endif
#if defined(HAVE_SYS_TYPES_H)
# include <sys/types.h>
#endif
#if defined(HAVE_SYS_SOCKET_H)
# include <sys/socket.h>
#endif
#if defined(HAVE_NETDB_H)
# include <netdb.h>
#endif
#if defined(HAVE_SYS_UN_H)
# include <sys/un.h>
#endif
#if defined(HAVE_NETINET_IN_H)
# include <netinet/in.h>
#endif

#include "intl.h"
#include "alternative_strlcpy.h"
#include "logger.h"
#include "cannaprotocol_common.h"
#include "cannaprotocol_low.h"
#include "cannaprotocol.h"



/** ƥȤ򳫻
 *@param[in,out]	cpt					̿Ѿ
 *@retval			NULL				顼
 *@return								̿Ѿ
 *
 *@author	G-HAL
 *@date		Sat,24 Aug,2013 - Sun,25 Aug,2013, Wed,28 Aug,2013, Sat,31 Aug,2013
 *@date		Mon,09 Sep,2013
 */
struct canna_protocol_context_transaction_t* init_canna_context(struct canna_protocol_transaction_t* const cpt)
{
	assert (NULL != cpt);


	/* Τν */

	struct canna_protocol_context_transaction_t* const cpct = malloc(sizeof(*cpct));
	if (NULL == cpct) {
		LOGGER_CRIT(l,N_("Out of memory at %1$s"), "init_canna_context()");
		return NULL;
	}
	{
		cpct->cpt = cpt;
		cpct->next = cpt->list_cpct;
		cpct->context = -1;
		cpct->dic.num = 0;
		cpct->dic.list[0] = '\0';
		cpct->dic.list[1] = '\0';
	}


	/* ̿ */
	if (!cannaprotocol_CreateContext(cpct)) {
		goto ERR_FREE;
	}

	{	/* ޥ */
		if (!cannaprotocol_GetDictionaryList(cpct)) {
			_unused_ const bool ret_cls = cannaprotocol_CloseContext(cpct);
			goto ERR_FREE;
		}

		const string8_t* ptr = cpct->dic.list;
		for (int_fast16_t dic_ptr = 0; dic_ptr < cpct->dic.num; ++dic_ptr) {
			const size_t len_dicname = string8len(ptr);
			if (len_dicname < 1) {
				LOGGER_EMERG(l,N_("Data list was broken at %1$s"), "init_canna_context()");
				break;
			}

			const bool ret_mnt = cannaprotocol_MountDictionary(cpct, ptr);
			if (!ret_mnt) {
				_unused_ const bool ret_cls = cannaprotocol_CloseContext(cpct);
				goto ERR_FREE;
			}
			ptr += (len_dicname + 1);
		}
	}

	cpt->list_cpct = cpct;
	return cpct;


ERR_FREE:
	free(cpct);
	return NULL;
}


/** ФȤ̿򳫻
 *@param			domain				̿ΥɥᥤPF_UNIX  PF_INET Τб
 *@param[in]		name				PF_UNIX: å̾PF_INET: ̾
 *@param[in]		serv				PF_INET: ФΥӥ̾
 *@param[in]		group				롼̾NULL: ꤷʤ
 *@param[in]		user				桼̾
 *@param[in]		locale				̾NULL: ꤷʤ
 *@retval			NULL				顼
 *@return								̿Ѿ
 *
 *@attention
 *		󥹥åɥմؿѡ
 *
 *@author	G-HAL
 *@date		Sat,24 Aug,2013 - Sun,25 Aug,2013, Wed,28 Aug,2013, Sat,31 Aug,2013
 *@date		Mon,09 Sep,2013
 *@date		Sun,22 Sep,2013
 *@date		Mon,30 Sep,2013
 *@date		Sat,04 Feb,2017
 */
struct canna_protocol_transaction_t* init_canna(int domain, const char name[], const char serv[], const char group[], const char user[], const char locale[])
{
	assert (NULL != name);
	assert (NULL != serv);
	/* assert (NULL != group); */
	assert (NULL != user);


	/* Τν */
	struct canna_protocol_transaction_t* const cpt = malloc(sizeof(*cpt));
	if (NULL == cpt) {
		LOGGER_CRIT(l,N_("Out of memory at %1$s"), "init_canna()");
		return NULL;
	}
	{
		string8lcpy(cpt->cannaprotocol_version, (const string8_t* const)CANNAPROTOCOL_VERSION, (uint_fast16_t)number_(cpt->cannaprotocol_version));
		cpt->initial_context = -1;
		cpt->list_cpct = NULL;
	}


	/* ³ */
	switch (domain) {
	case PF_UNIX:
		{
			cpt->sock = socket(domain, SOCK_STREAM, 0);
			if (-1 == cpt->sock) {
				LOGGER_ERR(l,N_("Can not open socket(%2$s) at %1$s %3$s"), "init_canna()", "PF_UNIX", strerror(errno));
				goto ERR_FREE;
			}

			struct sockaddr_un remote;
		  #if defined(__FreeBSD__) || defined(__NetBSD__) || defined(__OpenBSD__)
			remote.sun_len = 0;		/* Dummy */
		  #endif
			remote.sun_family = AF_UNIX;
			strlcpy(remote.sun_path, name, number_(remote.sun_path));

			const int ret_connect = connect(cpt->sock, (const struct sockaddr *)&remote, (socklen_t)sizeof(remote));
			if (0 != ret_connect) {
				LOGGER_ERR(l,N_("Can not connect socket(%2$s) at %1$s. %3$s"), "init_canna()", "PF_UNIX", strerror(errno));
				goto ERR_FREE;
			}
		}
		break;
	case PF_INET:
		{
			const struct hostent* const remoteinfo = gethostbyname(name);
			if (NULL == remoteinfo) {
				LOGGER_ERR(l,N_("Can not get host by name at %1$s %2$s"), "init_canna()", strerror(errno));
				return NULL;
			}
			const struct servent* const servinfo = getservbyname(serv, DEFAULT_SOCKET_PROTOCOL);
			if (NULL == servinfo) {
				LOGGER_ERR(l,N_("Can not get serv by name at %1$s %2$s"), "init_canna()", strerror(errno));
				return NULL;
			}

			cpt->sock = socket(domain, SOCK_STREAM, 0);
			if (-1 == cpt->sock) {
				LOGGER_ERR(l,N_("Can not open socket(%2$s) at %1$s %3$s"), "init_canna()", "PF_INET", strerror(errno));
				goto ERR_FREE;
			}

			struct sockaddr_in remote;
		  #if defined(__FreeBSD__) || defined(__NetBSD__) || defined(__OpenBSD__)
			remote.sin_len = 0;		/* Dummy */
		  #endif
			remote.sin_family = AF_INET;
			remote.sin_addr = *(struct in_addr *)*remoteinfo->h_addr_list;
			remote.sin_port = (in_port_t)servinfo->s_port;

			const int ret_connect = connect(cpt->sock, (const struct sockaddr *)&remote, (socklen_t)sizeof(remote));
			if (0 != ret_connect) {
				LOGGER_ERR(l,N_("Can not connect socket(%2$s) at %1$s. %3$s"), "init_canna()", "PF_INET", strerror(errno));
				goto ERR_FREE;
			}
		}
		break;
	default:
		goto ERR_FREE;
	}


	/* ̿ */
	if (!cannaprotocol_Initialize(cpt, (const string8_t* const)user)) {
		goto ERR_FREE;
	}
	if (!cannaprotocol_NoticeGroupName(cpt, (const string8_t* const)group)) {
		goto ERR_FREE;
	}
	if (!cannaprotocol_SetLocale(cpt, (const string8_t* const)locale)) {
		goto ERR_FREE;
	}
	return cpt;


ERR_FREE:
	free(cpt);
	return NULL;
}



/** ƥȤλ
 *@param[in,out]	cpct				̿Ѿ
 *@retval			true				
 *@retval			false				
 *
 *@author	G-HAL
 *@date		Sat,24 Aug,2013
 *@date		Mon,09 Sep,2013
 */
bool fin_canna_context(struct canna_protocol_context_transaction_t* const cpct)
{
	assert (NULL != cpct);

	bool ret = false;

	/* ƥȽλ */
	if (!cannaprotocol_CloseContext(cpct)) {
		ret = false;
	}

	/* ƥȤտ魯 */
	bool hit = false;
	struct canna_protocol_context_transaction_t** prev_ptr = &(cpct->cpt->list_cpct);
	for (struct canna_protocol_context_transaction_t* ptr = cpct->cpt->list_cpct; NULL != ptr; prev_ptr = &(ptr->next), ptr = ptr->next) {
		if (cpct == ptr) {
			hit = true;
			(*prev_ptr) = ptr->next;
			break;
		}
	}
	if (hit) {
		ret = true;
		free(cpct);
	} else {
		ret = false;
		LOGGER_EMERG(l,N_("Data list was broken at %1$s"), "fin_canna_context()");
	}
	return ret;
}


/** ФȤ̿λ
 *@param[in,out]	cpt					̿Ѿ
 *@retval			true				
 *@retval			false				
 *
 *@author	G-HAL
 *@date		Sat,24 Aug,2013
 *@date		Mon,09 Sep,2013
 */
bool fin_canna(struct canna_protocol_transaction_t* const cpt)
{
	assert (NULL != cpt);

	bool ret = true;

	/* ƥȲ */
	for (struct canna_protocol_context_transaction_t* ptr = cpt->list_cpct; NULL != ptr;) {
		struct canna_protocol_context_transaction_t* const next_ptr = ptr->next;
		const bool ret_fin_cc = fin_canna_context(ptr);
		if (!ret_fin_cc) {
			ret = false;
			break;
		}
		ptr = next_ptr;
	}

	/* ̿λ */
	if (!cannaprotocol_Finalize(cpt)) {
		ret = false;
	}

	/* ³ */
	const int ret_close = close(cpt->sock);
	if (0 != ret_close) {
		LOGGER_ERR(l,N_("Can not close socket at %1$s %2$s"), "fin_canna()", strerror(errno));
	}

	/* Τθ */
	free(cpt);

	return ret;
}

/* [ End of File ] */
