/**@file
 *@brief	cannaץȥ򤷤٤ؿ顧ƥȽ
 *@date		Fri,23 Aug,2013 - Sun,22 Sep,2013
 *@author	Copyright(C)2013 G-HAL, All rights reserved.
 *
 *@note
 *@verbatim
Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions
are met:

* Redistributions of source code must retain the above copyright
notice, this list of conditions and the following disclaimer.

* Redistributions in binary form must reproduce the above copyright
notice, this list of conditions and the following disclaimer in the
documentation and/or other materials provided with the distribution.

* Neither the name of the copyright holders nor the names of its contributors
may be used to endorse or promote products derived from this software
without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
"AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE COPYRIGHT OWNER OR
CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *@endverbatim
 */

#if defined(HAVE_CONFIG_H)
# include "config.h"
#endif

#if defined(HAVE_STDLIB_H)
# include <stdlib.h>
#endif
#if defined(HAVE_STDIO_H)
# include <stdio.h>
#endif
#if defined(HAVE_STDBOOL_H)
# include <stdbool.h>
#endif

#if defined(HAVE_ASSERT_H)
# include <assert.h>
#endif

#if !defined(__STDC_LIMIT_MACROS)
# define __STDC_LIMIT_MACROS
#endif
#if !defined(__STDC_CONSTANT_MACROS)
# define __STDC_CONSTANT_MACROS
#endif
#if defined(HAVE_STDINT_H)
# include <stdint.h>
#endif
#if !defined(__STDC_FORMAT_MACROS)
# define __STDC_FORMAT_MACROS
#endif
#if defined(HAVE_INTTYPES_H)
# include <inttypes.h>
#endif

#if defined(HAVE_ERRNO_H)
# include <errno.h>
#endif
#if defined(HAVE_STRING_H)
# include <string.h>
#endif
#if defined(HAVE_NETINET_IN_H)
# include <netinet/in.h> /* for OpenBSD, netinet/in.h must faster than arpa/inet.h */
#endif
#if defined(HAVE_ARPA_INET_H)
# include <arpa/inet.h>
#endif
#if defined(HAVE_SYS_TYPES_H)
# include <sys/types.h>
#endif
#if defined(HAVE_SYS_UIO_H)
# include <sys/uio.h>
#endif
#if defined(HAVE_UNISTD_H)
# include <unistd.h>
#endif

#include "intl.h"
#include "logger.h"
#include "type_punned_cast.h"
#include "cannaprotocol_common.h"
#include "cannaprotocol_low_sub.h"
#include "cannaprotocol_low.h"



/** ʥץȥ롧ƥȺ
 *@param[in,out]	cpct					̿Ѿ
 *@retval			true				
 *@retval			false				
 *
 *@author	G-HAL
 *@date		Sat,24 Aug,2013
 *@date		Mon,09 Sep,2013
 */
bool cannaprotocol_CreateContext(struct canna_protocol_context_transaction_t* const cpct)
{
	assert (NULL != cpct);

	{	/*  */
		size_t len_write;
		void* ptr_write = cpct->buf.w;
		*((union type_punned_pointer_cast_t* const)&ptr_write)->u8p ++ = CannaProtocol_CreateContext;
		*((union type_punned_pointer_cast_t* const)&ptr_write)->u8p ++ = UINT8_C(0x00);
		*((union type_punned_pointer_cast_t* const)&ptr_write)->u16p ++ = htons(0);
		len_write = 4;
		assert (len_write <= number_(cpct->buf.w));
		ssize_t ret_write = write(cpct->cpt->sock, cpct->buf.w, len_write);
		if (-1 == ret_write) {
			LOGGER_ERR(l,N_("write() failed at %1$s %2$s"), "cannaprotocol_CreateContext()", strerror(errno));
			return false;
		}
		if (ret_write != (ssize_t)len_write) {
			LOGGER_ERR(l,N_("write() mismatch at %1$s %2$s"), "cannaprotocol_CreateContext()", strerror(errno));
		}
	}
	{	/*  */
		const ssize_t len_read = cannaprotocol_check_result(cpct->cpt, CannaProtocol_CreateContext, cpct->buf.r, (uint_fast16_t)number_(cpct->buf.r));
		void* ptr_read = &(cpct->buf.r[4]);
		if (6 != len_read) {
			LOGGER_ERR(l,N_("The reply size of the server is mismatch at %1$s %2$zd"), "cannaprotocol_CreateContext()", len_read);
			return false;
		}
		const int_fast16_t dat_context = (int_fast16_t)ntohs(*((union type_punned_pointer_cast_t* const)&ptr_read)->u16p ++);
		if (-1 == dat_context) {
			LOGGER_ERR(l,N_("Can not get context at %1$s"), "cannaprotocol_CreateContext()");
			return false;
		}
		cpct->context = dat_context;
	}
	return true;
}



/** ʥץȥ롧ƥȺ
 *@param[in,out]	cpct					̿Ѿ
 *@retval			true				
 *@retval			false				
 *
 *@author	G-HAL
 *@date		Sat,24 Aug,2013
 *@date		Mon,09 Sep,2013
 */
bool cannaprotocol_CloseContext(struct canna_protocol_context_transaction_t* const cpct)
{
	assert (NULL != cpct);

	{	/*  */
		size_t len_write;
		void* ptr_write = cpct->buf.w;
		*((union type_punned_pointer_cast_t* const)&ptr_write)->u8p ++ = CannaProtocol_CloseContext;
		*((union type_punned_pointer_cast_t* const)&ptr_write)->u8p ++ = UINT8_C(0x00);
		*((union type_punned_pointer_cast_t* const)&ptr_write)->u16p ++ = htons(2);
		*((union type_punned_pointer_cast_t* const)&ptr_write)->u16p ++ = htons((uint16_t)cpct->context);
		len_write = 6;
		assert (len_write <= number_(cpct->buf.w));
		ssize_t ret_write = write(cpct->cpt->sock, cpct->buf.w, len_write);
		if (-1 == ret_write) {
			LOGGER_ERR(l,N_("write() failed at %1$s %2$s"), "cannaprotocol_CloseContext()", strerror(errno));
			return false;
		}
		if (ret_write != (ssize_t)len_write) {
			LOGGER_ERR(l,N_("write() mismatch at %1$s %2$s"), "cannaprotocol_CloseContext()", strerror(errno));
		}
	}
	{	/*  */
		const ssize_t len_read = cannaprotocol_check_result(cpct->cpt, CannaProtocol_CloseContext, cpct->buf.r, (uint_fast16_t)number_(cpct->buf.r));
		void* ptr_read = &(cpct->buf.r[4]);
		if (5 != len_read) {
			LOGGER_ERR(l,N_("The reply size of the server is mismatch at %1$s %2$zd"), "cannaprotocol_CloseContext()", len_read);
			return false;
		}
		const int_fast8_t dat_stat = *((union type_punned_pointer_cast_t* const)&ptr_read)->s8p ++;
		switch (dat_stat) {
		case 0:
			break;
		case -1:
			LOGGER_ERR(l,N_("Failed at %1$s."), "cannaprotocol_CloseContext()");
			return false;
		default:
			LOGGER_ERR(l,N_("The reply of the server is unknown at %1$s: buf[4] = %2$"PRId8""), "cannaprotocol_CloseContext()", (int8_t)dat_stat);
			return false;
		}
	}
	return true;
}

/* [ End of File ] */
