/**@file
 *@brief	cannaץȥ򤷤٤ؿ顧ʻμ
 *@date		Sun,08 Sep,2013 - Sun,22 Sep,2013
 *@author	Copyright(C)2013 G-HAL, All rights reserved.
 *
 *@note
 *@verbatim
Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions
are met:

* Redistributions of source code must retain the above copyright
notice, this list of conditions and the following disclaimer.

* Redistributions in binary form must reproduce the above copyright
notice, this list of conditions and the following disclaimer in the
documentation and/or other materials provided with the distribution.

* Neither the name of the copyright holders nor the names of its contributors
may be used to endorse or promote products derived from this software
without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
"AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE COPYRIGHT OWNER OR
CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *@endverbatim
 */

#if defined(HAVE_CONFIG_H)
# include "config.h"
#endif

#if defined(HAVE_STDLIB_H)
# include <stdlib.h>
#endif
#if defined(HAVE_STDIO_H)
# include <stdio.h>
#endif
#if defined(HAVE_STDBOOL_H)
# include <stdbool.h>
#endif

#if defined(HAVE_ASSERT_H)
# include <assert.h>
#endif

#if !defined(__STDC_LIMIT_MACROS)
# define __STDC_LIMIT_MACROS
#endif
#if !defined(__STDC_CONSTANT_MACROS)
# define __STDC_CONSTANT_MACROS
#endif
#if defined(HAVE_STDINT_H)
# include <stdint.h>
#endif
#if !defined(__STDC_FORMAT_MACROS)
# define __STDC_FORMAT_MACROS
#endif
#if defined(HAVE_INTTYPES_H)
# include <inttypes.h>
#endif

#if defined(HAVE_ERRNO_H)
# include <errno.h>
#endif
#if defined(HAVE_STRING_H)
# include <string.h>
#endif
#if defined(HAVE_NETINET_IN_H)
# include <netinet/in.h> /* for OpenBSD, netinet/in.h must faster than arpa/inet.h */
#endif
#if defined(HAVE_ARPA_INET_H)
# include <arpa/inet.h>
#endif

#include "intl.h"
#include "logger.h"
#include "type_punned_cast.h"
#include "cannaprotocol_common.h"
#include "cannaprotocol_low_sub.h"
#include "cannaprotocol_low.h"



/** ʥץȥ롧ʻμ
 *@param[in,out]	cpct					̿Ѿ
 *@param			cur_phrase			ʸֹ
 *@param			cur_candidacy		ֹ
 *@param[out]		len_hinshi			ʻĹ
 *@param[out]		hinshi				ʻ
 *@param			maxlen_hinshi		hinshi ˳ǼǤʸĹ
 *@retval			true				
 *@retval			false				
 *
 *@author	G-HAL
 *@date		Sun,08 Sep,2013
 */
bool cannaprotocol_GetHinshi(struct canna_protocol_context_transaction_t* const cpct, int_fast16_t cur_phrase, int_fast16_t cur_candidacy, int_fast16_t* const len_hinshi, string16_t* const hinshi, size_t maxlen_hinshi)
{
	assert (NULL != cpct);
	assert (NULL != len_hinshi);
	assert (NULL != hinshi);

	{	/*  */
		void* ptr_write = cpct->buf.w;
		*((union type_punned_pointer_cast_t* const)&ptr_write)->u8p ++ = CannaProtocol_GetHinshi;
		*((union type_punned_pointer_cast_t* const)&ptr_write)->u8p ++ = UINT8_C(0x00);
		const size_t len_data = 8;
		*((union type_punned_pointer_cast_t* const)&ptr_write)->u16p ++ = htons((uint16_t)len_data);
		*((union type_punned_pointer_cast_t* const)&ptr_write)->s16p ++ = (int16_t)htons((uint16_t)cpct->context);
		*((union type_punned_pointer_cast_t* const)&ptr_write)->s16p ++ = (int16_t)htons((uint16_t)cur_phrase);
		*((union type_punned_pointer_cast_t* const)&ptr_write)->s16p ++ = (int16_t)htons((uint16_t)cur_candidacy);
		*((union type_punned_pointer_cast_t* const)&ptr_write)->u16p ++ = htons(number_(cpct->buf.w) - 8);
		const size_t len_write = (4 + len_data);
		assert (len_write <= number_(cpct->buf.w));
		ssize_t ret_write = write(cpct->cpt->sock, cpct->buf.w, len_write);
		if (-1 == ret_write) {
			LOGGER_ERR(l,N_("write() failed at %1$s %2$s"), "cannaprotocol_GetHinshi()", strerror(errno));
			return false;
		}
		if (ret_write != (ssize_t)len_write) {
			LOGGER_ERR(l,N_("write() mismatch at %1$s %2$s"), "cannaprotocol_GetHinshi()", strerror(errno));
		}
	}
	{	/*  */
		const ssize_t len_read = cannaprotocol_check_result(cpct->cpt, CannaProtocol_GetHinshi, cpct->buf.r, (uint_fast16_t)number_(cpct->buf.r));
		void* ptr_read = &(cpct->buf.r[4]);
		if (len_read < 7) {
			LOGGER_ERR(l,N_("The reply size of the server is mismatch at %1$s %2$zd"), "cannaprotocol_GetHinshi()", len_read);
			return false;
		}
		const int_fast16_t dat_stat = (int_fast16_t)ntohs(*((union type_punned_pointer_cast_t* const)&ptr_read)->u16p ++);
		if (-1 == dat_stat) {
			LOGGER_ERR(l,N_("Failed at %1$s."), "cannaprotocol_GetHinshi()");
			return false;
		}

		const size_t dat_len_hinshi = (size_t)(len_read - 5);
		if (NULL != len_hinshi) {
			(*len_hinshi) = (int_fast16_t)(dat_len_hinshi - 2) / 2;
		}
		if (NULL != hinshi) {
			if (maxlen_hinshi < dat_len_hinshi) {
				LOGGER_ERR(l,N_("Buffer overflow at %1$s"), "cannaprotocol_GetHinshi()");
				return false;
			}
			memcpy(hinshi, ptr_read, (sizeof(hinshi[0]) * dat_len_hinshi));
		}
	}
	return true;
}

/* [ End of File ] */
