/**@file
 *@brief	cannaץȥ򤷤٤ؿ顧ʬ
 *@date		Fri,23 Aug,2013 - Sun,22 Sep,2013
 *@author	Copyright(C)2013 G-HAL, All rights reserved.
 *
 *@note
 *@verbatim
Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions
are met:

* Redistributions of source code must retain the above copyright
notice, this list of conditions and the following disclaimer.

* Redistributions in binary form must reproduce the above copyright
notice, this list of conditions and the following disclaimer in the
documentation and/or other materials provided with the distribution.

* Neither the name of the copyright holders nor the names of its contributors
may be used to endorse or promote products derived from this software
without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
"AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE COPYRIGHT OWNER OR
CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *@endverbatim
 */

#if defined(HAVE_CONFIG_H)
# include "config.h"
#endif

#if defined(HAVE_STDLIB_H)
# include <stdlib.h>
#endif
#if defined(HAVE_STDIO_H)
# include <stdio.h>
#endif
#if defined(HAVE_STDBOOL_H)
# include <stdbool.h>
#endif

#if defined(HAVE_ASSERT_H)
# include <assert.h>
#endif

#if !defined(__STDC_LIMIT_MACROS)
# define __STDC_LIMIT_MACROS
#endif
#if !defined(__STDC_CONSTANT_MACROS)
# define __STDC_CONSTANT_MACROS
#endif
#if defined(HAVE_STDINT_H)
# include <stdint.h>
#endif

#if defined(HAVE_STRING_H)
# include <string.h>
#endif
#if defined(HAVE_ERRNO_H)
# include <errno.h>
#endif
#if defined(HAVE_SYS_TYPES_H)
# include <sys/types.h>
#endif
#if defined(HAVE_SYS_UIO_H)
# include <sys/uio.h>
#endif
#if defined(HAVE_UNISTD_H)
# include <unistd.h>
#endif
#if defined(HAVE_NETINET_IN_H)
# include <netinet/in.h> /* for OpenBSD, netinet/in.h must faster than arpa/inet.h */
#endif
#if defined(HAVE_ARPA_INET_H)
# include <arpa/inet.h>
#endif

#include "intl.h"
#include "logger.h"
#include "type_punned_cast.h"
#include "cannaprotocol_common.h"
#include "cannaprotocol_low_sub.h"



/** ʥץȥ롧ʬθ
 *@param[in,out]	cpt					̿Ѿ
 *@param			arg_code			Ԥ᥸㡼ɤ
 *@param[in,out]	ptr					ɤ߹
 *@param			buf_size			ɤ߹ɤ߹Ĺ
 *@retval			-1					Թ
 *@return								ɤ߹ǡĹ
 *
 *@author	G-HAL
 *@date		Sat,24 Aug,2013 - Mon,26 Aug,2013, Wed,28 Aug,2013, Fri,30 Aug,2013
 */
ssize_t cannaprotocol_check_result(struct canna_protocol_transaction_t* const cpt, uint_fast8_t arg_code, void* ptr, uint_fast16_t buf_size)
{
	assert (NULL != cpt);
	assert (NULL != ptr);
	if (buf_size < 4) {
		LOGGER_ERR(l,N_("Buffer size is too small at %1$s. %2$"PRIu16""), "cannaprotocol_check_result()", (uint16_t)buf_size);
		return -1;
	}

	const ssize_t headerlen_read = read(cpt->sock, ptr, (size_t)4);
	if (-1 == headerlen_read) {
		LOGGER_ERR(l,N_("read() failed at %1$s. %2$s"), "cannaprotocol_check_result()", strerror(errno));
		return -1;
	} else if (4 != headerlen_read) {
		LOGGER_ERR(l,N_("read() failed, size mismatch at %1$s. %2$zd"), "cannaprotocol_check_result()", headerlen_read);
		return -1;
	}
	const uint_fast8_t dat_major_code = *((union type_punned_pointer_cast_t* const)&ptr)->u8p ++;
	if (arg_code != dat_major_code) {
		LOGGER_ERR(l,N_("The reply of the server is mismatch at %1$s: buf[0] = %2$02"PRIX8""), "cannaprotocol_check_result()", dat_major_code);
		return -1;
	}
	const uint_fast8_t dat_minor_code = *((union type_punned_pointer_cast_t* const)&ptr)->u8p ++;
	if (UINT8_C(0x00) != dat_minor_code) {
		LOGGER_ERR(l,N_("The reply of the server is mismatch at %1$s: buf[1] = %2$02"PRIX8""), "cannaprotocol_check_result()", dat_minor_code);
		return -1;
	}
	const uint_fast16_t dat_datalen = ntohs(*((union type_punned_pointer_cast_t* const)&ptr)->u16p ++);
	if ((buf_size - 4) < dat_datalen) {
		LOGGER_ERR(l,N_("Buffer size is mismatch at %1$s: buf[2] = %2$04"PRIX16""), "cannaprotocol_check_result()", (uint16_t)dat_datalen);
		return -1;
	}

	const ssize_t datalen_read = read(cpt->sock, ptr, (size_t)dat_datalen);
	if (datalen_read != (ssize_t)dat_datalen) {
		LOGGER_ERR(l,N_("The reply size of the server is mismatch at %1$s: read() = %2$04"PRIX16""), "cannaprotocol_check_result()", (uint16_t)dat_datalen);
		return -1;
	}
	return (headerlen_read + datalen_read);
}

/* [ End of File ] */
