/**@file
 *@brief	anthy-agentץȥʹؿ磻ɥ - EUC-JP Ѵ
 *@date		Mon,02 Sep,2013 - Sun,08 Sep,2013
 *@date		Mon,30 Sep,2013
 *@date		Sat,09 Nov,2013
 *@author	Copyright(C)2013 G-HAL, All rights reserved.
 *
 *@note
 *@verbatim
Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions
are met:

* Redistributions of source code must retain the above copyright
notice, this list of conditions and the following disclaimer.

* Redistributions in binary form must reproduce the above copyright
notice, this list of conditions and the following disclaimer in the
documentation and/or other materials provided with the distribution.

* Neither the name of the copyright holders nor the names of its contributors
may be used to endorse or promote products derived from this software
without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
"AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE COPYRIGHT OWNER OR
CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *@endverbatim
 */
#ifndef cannaprotocol_wchar_h_included_
#define cannaprotocol_wchar_h_included_

#if defined(HAVE_CONFIG_H)
# include "config.h"
#endif

#if defined(HAVE_LIMITS_H)
# include <limits.h>
#endif
#if (CHAR_BIT != 8)
# error "This program requires that char bit is 8."
#endif

#if !defined(__STDC_LIMIT_MACROS)
# define __STDC_LIMIT_MACROS
#endif
#if !defined(__STDC_CONSTANT_MACROS)
# define __STDC_CONSTANT_MACROS
#endif
#if defined(HAVE_STDINT_H)
# include <stdint.h>
#endif

#include "cannaprotocol_common.h"



#ifdef __cplusplus
extern "C" {
#endif



/** EUC-JP G0 ASCII 磻ɥ饯Ѵ
 *@param			c0					Ѵ EUC-JP Х
 *@return								磻ɥ饯
 *
 *@author	G-HAL
 *@date		Mon,02 Sep,2013
 */
inline static string16_t cannaprotocol_eucjpG0_to_wchar(uint_fast8_t c0) {
	return UINT16_C(0x0000) | (UINT16_C(0x007F) & c0);
}

/** EUC-JP G1 JIS X 0208 磻ɥ饯Ѵ
 *@param			c0					Ѵ EUC-JP Х
 *@param			c1					Ѵ EUC-JP Х
 *@return								磻ɥ饯
 *
 *@author	G-HAL
 *@date		Mon,02 Sep,2013
 */
inline static string16_t cannaprotocol_eucjpG1_to_wchar(uint_fast8_t c0, uint_fast8_t c1) {
	return (string16_t)(UINT16_C(0x8080) | (UINT16_C(0x7F7F) & ((((string16_t)c0) << 8) | ((string16_t)c1))));
}

/** EUC-JP G2 JIS X 0201 磻ɥ饯Ѵ
 *@param			c1					Ѵ EUC-JP Х
 *@return								磻ɥ饯
 *
 *@author	G-HAL
 *@date		Mon,02 Sep,2013
 */
inline static string16_t cannaprotocol_eucjpG2_to_wchar(uint_fast8_t c1) {
	return UINT16_C(0x0080) | (UINT16_C(0x007F) & ((string16_t)c1));
}

/** EUC-JP G3 JIS X 0212 磻ɥ饯Ѵ
 *@param			c1					Ѵ EUC-JP Х
 *@param			c2					Ѵ EUC-JP Х
 *@return								磻ɥ饯
 *
 *@author	G-HAL
 *@date		Mon,02 Sep,2013
 */
inline static string16_t cannaprotocol_eucjpG3_to_wchar(uint_fast8_t c1, uint_fast8_t c2) {
	return (string16_t)(UINT16_C(0x8000) | (UINT16_C(0x7F7F) & ((((string16_t)c1) << 8) | ((string16_t)c2))));
}

/** ASCII ɤȽ
 *@param			c					ѴΥޥХʸ
 *@retval			true				ASCII Ǥ롣
 *@retval			false				EUC-JP ⤷̤ʸɤǤ롣
 *
 *@author	G-HAL
 *@date		Thu,05 Sep,2013
 *@date		Sat,09 Nov,2013
 */
inline static bool cannaprotocol_is_ascii(string_internal_fast_t c) {
	if (c <= UINT32_C(0x0000007F)) {
		return true;
	}
	return false;
}

/** EUC-JP μ̤Ƚ
 *@param			c					ѴΥޥХʸ
 *@param			c0					Ѵβ̥Х
 *@param			c1					Ѵ̥Х
 *@param			c2					Ѵξ̥Х
 *@retval			-1					̤μ
 *@retval			0					G0
 *@retval			1					G1
 *@retval			2					G2
 *@retval			3					G3
 *
 *@author	G-HAL
 *@date		Thu,05 Sep,2013
 *@date		Sat,09 Nov,2013
 */
inline static int cannaprotocol_is_eucjp(string_internal_fast_t c, uint_fast8_t c0, uint_fast8_t c1, uint_fast8_t c2) {
	if (cannaprotocol_is_ascii(c)) {
		return 0;
	} else if (c <= UINT32_C(0x00007FFF)) {
		return -1;
	} else if (c <= UINT32_C(0x0000FFFF)) {
		if (UINT8_C(0x00) == (UINT8_C(0x80) & c0)) {
			return -1;
		}
		if (UINT8_C(0x8E) == c1) {
			return 2;
		}
		return 1;
	} else if (0 != (c >> 24)) {
		return -1;
	} else if (UINT8_C(0x8F) == c2) {
		return 3;
	}
	return -1;
}

/** EUC-JP μ̤Ƚ̤ʸ狼ɤȽꤷʤ
 *@param			c0					ѴƬХ
 *@param			c1					Ѵ̥Х
 *@param			c2					ѴθХ
 *@retval			-1					̤μ
 *@retval			0					G0
 *@retval			1					G1
 *@retval			2					G2
 *@retval			3					G3
 *
 *@author	G-HAL
 *@date		Thu,05 Sep,2013
 */
inline static int cannaprotocol_is_char_eucjp_fast(uint_fast8_t c0, uint_fast8_t c1, uint_fast8_t c2) {
	if ((UINT8_C(0x8E) == c0)
		&& ((UINT8_C(0x80) <= c1))
	) {	/* EUC-JP G2 JIS X 0201 */
		return 2;

	} else if ((UINT8_C(0x8F) == c0)
			   && ((UINT8_C(0x80) <= c1))
			   && ((UINT8_C(0x80) <= c2))
	) {	/* EUC-JP G3 JIS X 0212 */
		return 3;

	} else if ((UINT8_C(0x80) <= c0)
			   && (UINT8_C(0x80) <= c1)
	) {	/* EUC-JP G1 JIS X0208 */
		return 1;

	}
	return 0;
}

/** EUC-JP 磻ɥ饯Ѵ̤ʸ狼ɤȽꤷʤ
 *@param			c					ѴΥޥХʸ
 *@return								ѴEUC-JP
 *
 *@author	G-HAL
 *@date		Mon,02 Sep,2013
 */
inline static string16_t cannaprotocol_eucjp_to_wchar_fast(string_internal_fast_t c) {
	const uint_fast8_t c0 = ((uint_fast8_t)(c >>  0) & UINT8_C(0xFF));
	const uint_fast8_t c1 = ((uint_fast8_t)(c >>  8) & UINT8_C(0xFF));
	const uint_fast8_t c2 = ((uint_fast8_t)(c >> 16) & UINT8_C(0xFF));
	if (c <= UINT32_C(0x000000FF)) {
		return cannaprotocol_eucjpG0_to_wchar(c0);
	} else if (UINT8_C(0x8E) == c1) {
		return cannaprotocol_eucjpG2_to_wchar(c0);
	} else if (c <= UINT32_C(0x0000FFFF)) {
		return cannaprotocol_eucjpG1_to_wchar(c1, c0);
	} else if (UINT8_C(0x8F) == c2) {
		return cannaprotocol_eucjpG3_to_wchar(c1, c0);
	}
	return cannaprotocol_eucjpG0_to_wchar(c0);
}


/** 磻ɥ饯 EUC-JP G0 ASCII Ѵ
 *@param			str					ѴΥ磻ɥ饯
 *@return								ѴEUC-JP
 *
 *@author	G-HAL
 *@date		Mon,02 Sep,2013
 */
inline static string_internal_fast_t cannaprotocol_wchar_to_eucjpG0(string16_t str) {
	return UINT8_C(0x7F) & str;
}

/** 磻ɥ饯 EUC-JP G1 JIS X 0208 Ѵ
 *@param			str					ѴΥ磻ɥ饯
 *@return								ѴEUC-JP
 *
 *@author	G-HAL
 *@date		Mon,02 Sep,2013
 */
inline static string_internal_fast_t cannaprotocol_wchar_to_eucjpG1(string16_t str) {
	return UINT16_C(0x8080) | (UINT16_C(0x7F7F) & str);
}

/** 磻ɥ饯 EUC-JP G2 JIS X 0201 Ѵ
 *@param			str					ѴΥ磻ɥ饯
 *@return								ѴEUC-JP
 *
 *@author	G-HAL
 *@date		Mon,02 Sep,2013
 */
inline static string_internal_fast_t cannaprotocol_wchar_to_eucjpG2(string16_t str) {
	return UINT16_C(0x8E80) | (UINT16_C(0x007F) & str);
}

/** 磻ɥ饯 EUC-JP G3 JIS X 0212 Ѵ
 *@param			str					ѴΥ磻ɥ饯
 *@return								ѴEUC-JP
 *
 *@author	G-HAL
 *@date		Mon,02 Sep,2013
 */
inline static string_internal_fast_t cannaprotocol_wchar_to_eucjpG3(string16_t str) {
	return UINT32_C(0x8F8080) | (UINT16_C(0x7F7F) & str);
}

/** 磻ɥ饯 EUC-JP Ѵ
 *@param			str					ѴΥ磻ɥ饯
 *@return								ѴEUC-JP
 *
 *@author	G-HAL
 *@date		Mon,02 Sep,2013
 */
inline static string_internal_fast_t cannaprotocol_wchar_to_eucjp(string16_t str) {
	const bool c0 = UINT16_C(0x8000) & str;
	const bool c1 = UINT16_C(0x0080) & str;
	if (!c0 && !c1) {
		return cannaprotocol_wchar_to_eucjpG0(str);
	} else if (c0 && c1) {
		return cannaprotocol_wchar_to_eucjpG1(str);
	} else if (!c0 && c1) {
		return cannaprotocol_wchar_to_eucjpG2(str);
	} else {
		return cannaprotocol_wchar_to_eucjpG3(str);
	}
}


/** EUC-JP G0 ASCII  string_internal_fast_t Ѵ
 *@param			c0					Ѵ EUC-JP Х
 *@return								磻ɥ饯
 *
 *@author	G-HAL
 *@date		Sun,08 Sep,2013
 */
inline static string_internal_fast_t cannaprotocol_eucjpG0_to_strinternal(uint_fast8_t c0) {
	return (UINT16_C(0x007F) & c0);
}

/** EUC-JP G1 JIS X 0208  string_internal_fast_t Ѵ
 *@param			c0					Ѵ EUC-JP Х
 *@param			c1					Ѵ EUC-JP Х
 *@return								磻ɥ饯
 *
 *@author	G-HAL
 *@date		Sun,08 Sep,2013
 */
inline static string_internal_fast_t cannaprotocol_eucjpG1_to_strinternal(uint_fast8_t c0, uint_fast8_t c1) {
	return (UINT16_C(0x8080) | (UINT16_C(0x7F7F) & ((((string_internal_fast_t)c0) << 8) | ((string_internal_fast_t)c1))));
}

/** EUC-JP G2 JIS X 0201  string_internal_fast_t Ѵ
 *@param			c1					Ѵ EUC-JP Х
 *@return								磻ɥ饯
 *
 *@author	G-HAL
 *@date		Sun,08 Sep,2013
 */
inline static string_internal_fast_t cannaprotocol_eucjpG2_to_strinternal(uint_fast8_t c1) {
	return UINT16_C(0x8E80) | (UINT16_C(0x007F) & ((string_internal_fast_t)c1));
}

/** EUC-JP G3 JIS X 0212  string_internal_fast_t Ѵ
 *@param			c1					Ѵ EUC-JP Х
 *@param			c2					Ѵ EUC-JP Х
 *@return								磻ɥ饯
 *
 *@author	G-HAL
 *@date		Sun,08 Sep,2013
 */
inline static string_internal_fast_t cannaprotocol_eucjpG3_to_strinternal(uint_fast8_t c1, uint_fast8_t c2) {
	return (UINT32_C(0x008F8080) | (UINT16_C(0x7F7F) & ((((string_internal_fast_t)c1) << 8) | ((string_internal_fast_t)c2))));
}



#ifdef __cplusplus
}
#endif

#endif /* cannaprotocol_wchar_h_included_ */
/* [ End of File ] */
