/**@file
 *@brief	ʸѴ
 *@date		Mon,16 Sep,2013 - Sat,09 Nov,2013
 *@date		Sun,09 Oct,2016 - Mon,10 Oct,2016
 *@author	Copyright(C)2009-2016 G-HAL, All rights reserved.
 *
 *@note
 *@verbatim
Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions
are met:

* Redistributions of source code must retain the above copyright
notice, this list of conditions and the following disclaimer.

* Redistributions in binary form must reproduce the above copyright
notice, this list of conditions and the following disclaimer in the
documentation and/or other materials provided with the distribution.

* Neither the name of the copyright holders nor the names of its contributors
may be used to endorse or promote products derived from this software
without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
"AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE COPYRIGHT OWNER OR
CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *@endverbatim
 */

#if defined(HAVE_CONFIG_H)
# include "config.h"
#endif

#if defined(HAVE_STDLIB_H)
# include <stdlib.h>
#endif
#if defined(HAVE_STDIO_H)
# include <stdio.h>
#endif
#if defined(HAVE_STDBOOL_H)
# include <stdbool.h>
#endif

#if defined(HAVE_ASSERT_H)
# include <assert.h>
#endif

#if defined(HAVE_ERRNO_H)
# include <errno.h>
#endif
#if !defined(__STDC_LIMIT_MACROS)
# define __STDC_LIMIT_MACROS
#endif
#if !defined(__STDC_CONSTANT_MACROS)
# define __STDC_CONSTANT_MACROS
#endif
#if defined(HAVE_STDINT_H)
# include <stdint.h>
#endif
#if defined(HAVE_STRING_H)
# include <string.h>
#endif

#if defined(HAVE_ICONV_H)
# include <iconv.h>
#endif
#if defined(HAVE_ICONV_OPEN) && !defined(HAVE_LIBICONV_OPEN)
# define libiconv_open iconv_open
#endif
#if defined(HAVE_ICONV) && !defined(HAVE_LIBICONV)
# define libiconv iconv
#endif
#if defined(HAVE_ICONV_CLOSE) && !defined(HAVE_LIBICONV_CLOSE)
# define libiconv_close iconv_close
#endif

#include "intl.h"
#include "args.h"
#include "logger.h"
#include "iconv_wrapper.h"
#include "parseline_rk.h"



/** 󥳡ǥ̾ʸֹѡˤѴơ֥Υǡå */
struct encoding_name2num_table_t {
	const char*	encoding_name;			/**< 󥳡ǥ̾ */
	enum ENCODING_NUM	encoding_num;	/**< ʸֹʥ󥳡ǥ̾ѡ */
};

/** 󥳡ǥ̾ʸֹѡˤѴơ֥ */
static const struct encoding_name2num_table_t encoding_name2num_table[] = {
	{ "",                       ENCODING_EUCJP0212, },
	/* iconv -l ١ */
	{ ""ICONV_EUCJP,            ENCODING_EUCJP0212, },
	{ ""ICONV_EUCJP0213,        ENCODING_EUCJP0213, },
	{ ""ICONV_EUCCN,            ENCODING_EUCCN, },
	{ ""ICONV_EUCTW,            ENCODING_EUCTW, },
	{ ""ICONV_BIG5,             ENCODING_BIG5, },
	{ ""ICONV_BIG5HKSCS,        ENCODING_BIG5HKSCS, },
	{ ""ICONV_EUCKR,            ENCODING_EUCKR, },
	{ ""ICONV_ISO2022JP,        ENCODING_ISO2022JP, },
	{ ""ICONV_SHIFTJIS,         ENCODING_SHIFTJIS, },
	{ ""ICONV_SHIFTJIS0213,     ENCODING_SHIFTJIS0213, },
	{ ""ICONV_SHIFTJISCP932,    ENCODING_SHIFTJISCP932, },
	{ ""ICONV_UTF8,             ENCODING_UTF8, },
	{ ""ICONV_UTF16BE,          ENCODING_UTF16BE, },
	{ ""ICONV_UTF32BE,          ENCODING_UTF32BE, },
	/* locale -m ١(FreeBSD) */
	{ "US-ASCII",               ENCODING_EUCJP0212, },
	{ "eucJP",                  ENCODING_EUCJP0212, },
	{ "eucCN",                  ENCODING_EUCCN, },
	{ "GB2312",                 ENCODING_EUCCN, },
	{ "Big5",                   ENCODING_BIG5, },
	{ "Big5HKSCS",              ENCODING_BIG5HKSCS, },
	{ "eucKR",                  ENCODING_EUCKR, },
	{ "UTF-8",                  ENCODING_UTF8, },
	{ "SJIS",                   ENCODING_SHIFTJIS, },
	/* locale -m ١(OpenBSD) */
	{ "646",                    ENCODING_EUCJP0212, },
	/* locale -m ١(GNU/Linux) */
	{ "EUC-JP",                 ENCODING_EUCJP0212, },
	{ "EUC-JISX0213",           ENCODING_EUCJP0213, }, /* ? */
	{ "GB2312",                 ENCODING_EUCCN, },
	{ "EUC-TW",                 ENCODING_EUCTW, },
	{ "BIG5",                   ENCODING_BIG5, },
	{ "BIG5-HKSCS",             ENCODING_BIG5HKSCS, },
	{ "EUC-KR",                 ENCODING_EUCKR, },
	{ "UTF-8",                  ENCODING_UTF8, },
	/* ʼ */
	{ NULL,                     ENCODING_MAX, },
};



/** 󥳡ǥѴѴơ֥Υǡå */
struct encoding_converter_table_t {
	bool                initialized;			/**< ѥե饰 */
	const char*         encoding_name_from;		/**< Ѵ󥳡ǥ̾ */
	const char*         encoding_name_to;		/**< Ѵ襨󥳡ǥ̾ */
	enum ENCODING_NUM   encoding_num_from;		/**< Ѵʸֹʥ󥳡ǥ̾ѡ */
	enum ENCODING_NUM   encoding_num_to;		/**< Ѵʸֹʥ󥳡ǥ̾ѡ */
	iconv_t             cd;						/**< Ѵǥץ */
	char                GETA[8+1];				/**< ʸѴʸɡ */
	size_t              len_GETA;				/**< ʸѴʸɡˤΥǡĹ */
};

/** 󥳡ǥѴѴơ֥ */
static struct encoding_converter_table_t converter_tbl[ENCODING_TABLE_MAX];



/** ޥХʸƬʸΥǡĹ
 *@param[in]			str						ʸ
 *@param				len_str					ʸĹ
 *@param				enc						ʸΥ󥳡ǥ
 *@return										ǡĹ
 *
 *@attention
 *		ʸξ 1 ֤
 *
 *@author	G-HAL
 *@date		Sun,07 Jun,2009 - Wed,10 Jun,2009
 *@date		Sat,15 Jun,2013
 *@date		Tue,17 Sep,2013
 *@date		Sat,09 Nov,2013
 *@date		Sun,09 Oct,2016 - Mon,10 Oct,2016
 */
size_t get_len_mchar(const char str[], size_t len_str, enum ENCODING_NUM enc)
{
	assert (NULL != str);

	size_t ret = 1;
	const uint8_t chr = (uint8_t)((1 <= len_str) ? str[0] : 0);
	switch (enc) {
	case ENCODING_ASCII:
		ret = 1;
		break;

	case ENCODING_EUCJP:
	case ENCODING_EUCJP0212:
	case ENCODING_EUCJP0213:
		if ((UINT8_C(0xA1) <= chr) && (chr <= UINT8_C(0xFE))) {
			ret = 2;
		} else if (UINT8_C(0x8E) == chr) {
			ret = 2;
		} else if (UINT8_C(0x8F) == chr) {
			ret = 3;
		} else {
			ret = 1;
		}
		break;

	case ENCODING_EUCCN:
		if ((UINT8_C(0xA1) <= chr) && (chr <= UINT8_C(0xFE))) {
			ret = 2;
		} else if (UINT8_C(0x8E) == chr) {
			ret = 1;	/* Illigal data. */
		} else if (UINT8_C(0x8F) == chr) {
			ret = 1;	/* Illigal data. */
		} else {
			ret = 1;
		}
		break;

	case ENCODING_EUCTW:
		if ((UINT8_C(0xA1) <= chr) && (chr <= UINT8_C(0xFE))) {
			ret = 2;
		} else if (UINT8_C(0x8E) == chr) {
			ret = 4;
		} else if (UINT8_C(0x8F) == chr) {
			ret = 1;	/* Illigal data. */
		} else {
			ret = 1;
		}
		break;

	case ENCODING_BIG5:
	case ENCODING_BIG5HKSCS:
		if ((UINT8_C(0xA1) <= chr) && (chr <= UINT8_C(0xC6))) {
			ret = 2;
		} else if ((UINT8_C(0xC9) <= chr) && (chr <= UINT8_C(0xF9))) {
			ret = 2;
		} else {
			ret = 1;
		}
		break;

	case ENCODING_EUCKR:
		if ((UINT8_C(0xA1) <= chr) && (chr <= UINT8_C(0xFE))) {
			ret = 2;
		} else if (UINT8_C(0x8E) == chr) {
			ret = 1;	/* Illigal data. */
		} else if (UINT8_C(0x8F) == chr) {
			ret = 1;	/* Illigal data. */
		} else {
			ret = 1;
		}
		break;

	case ENCODING_ISO2022JP:
		ret = 1;
		break;

	case ENCODING_SHIFTJIS:
		if ((UINT8_C(0x81) <= chr) && (chr <= UINT8_C(0x9F))) {
			ret = 2;
		} else if ((UINT8_C(0xE0) <= chr) && (chr <= UINT8_C(0xEF))) {
			ret = 2;
		} else {
			ret = 1;
		}
		break;

	case ENCODING_SHIFTJIS0213:
	case ENCODING_SHIFTJISCP932:
		if ((UINT8_C(0x81) <= chr) && (chr <= UINT8_C(0x9F))) {
			ret = 2;
		} else if ((UINT8_C(0xE0) <= chr) && (chr <= UINT8_C(0xFC))) {
			ret = 2;
		} else {
			ret = 1;
		}
		break;

	case ENCODING_UTF8:
		if (UINT8_C(0xC0) <= chr) {
			if ((UINT8_C(0xC0) <= chr) && (chr <= UINT8_C(0xDF))) {
				ret = 2;
			} else if ((UINT8_C(0xE0) <= chr) && (chr <= UINT8_C(0xEF))) {
				ret = 3;
			} else if ((UINT8_C(0xF0) <= chr) && (chr <= UINT8_C(0xF7))) {
				ret = 4;
			} else if ((UINT8_C(0xF8) <= chr) && (chr <= UINT8_C(0xFB))) {
				ret = 5;	/* Illigal data. */
			} else if ((UINT8_C(0xFC) <= chr) && (chr <= UINT8_C(0xFD))) {
				ret = 6;	/* Illigal data. */
			} else {
				ret = 1;	/* Broken data. UTF-16 BOM ? */
			}
		} else if ((UINT8_C(0x80) <= chr) && (chr <= UINT8_C(0xBF))) {
			ret = 1;	/* It is a trailer. */
		} else {
			ret = 1;
		}
		break;

	case ENCODING_UTF16BE:
		if ((UINT8_C(0xD8) <= chr) && (chr <= UINT8_C(0xDB))) {
			ret = 4;	/* High-Surrogate Code Point */
		} else if ((UINT8_C(0xDC) <= chr) && (chr <= UINT8_C(0xDF))) {
			ret = 2;	/* Low-Surrogate Code Point */
		} else {
			ret = 2;
		}
		break;

	case ENCODING_UTF32BE:
		ret = 4;
		break;

	case ENCODING_MAX:
		ret = 1;
		break;
	}
	return (ret < len_str) ? ret : len_str;
}



/** ʸʸɤѴ
 *@param			enc_tbl				ѤѴơֹ֥
 *@param[in,out]	dst					Ѵ
 *@param			maxlen_dst			dst γǼǽʸ
 *@param[in]		src					Ѵ
 *@return								dst ˳Ǽʸ
 *
 *@author	G-HAL
 *@date		Sun,07 Jun,2009 - Wed,10 Jun,2009
 *@date		Sun,14 Jun,2009 - Mon,15 Jun,2009
 *@date		Wed,18 Sep,2013
 */
size_t wrapper_iconv(enum ENCODING_TABLE enc_tbl, char* const dst, int16_t maxlen_dst, const char* const src, size_t len_src)
{
	if (ENCODING_TABLE_MAX <= enc_tbl) {
		LOGGER_CRIT(l,N_("Illigal argment at %1$s %2$u"), "wrapper_iconv()", enc_tbl);
		return 0;
	}
	assert (NULL != dst);
	assert (NULL != src);
	assert (0 < maxlen_dst);

	size_t       num_src = len_src;
	const char*  ptr_src = src;
	size_t       num_dst = (size_t)maxlen_dst;
	char*        ptr_dst = dst;
	{
		struct encoding_converter_table_t* const cvt = &(converter_tbl[enc_tbl]);
		{
			const size_t result = libiconv(cvt->cd, NULL, NULL, NULL, NULL);
			if ((size_t)(-1) != result) {
			} else {
				LOGGER_ERR(l,N_("State error at %1$s %2$zu"), "wrapper_iconv()", len_src);
			}
		}
		while (true) {
			const size_t result = libiconv(cvt->cd, &ptr_src, &num_src, &ptr_dst, &num_dst);
			if ((size_t)(-1) != result) {
				break;
			}
			/* ѴǤʤäʸˤ˥ʸͤùࡣ */
			{
				if (num_dst < cvt->len_GETA) {
					LOGGER_CRIT(l,N_("Out of memory at %1$s"), "wrapper_iconv()");
					break;
				}
				memcpy(ptr_dst, cvt->GETA, cvt->len_GETA);
				num_dst -= cvt->len_GETA;
				ptr_dst += cvt->len_GETA;
			}
			{
				const size_t len_wchar = get_len_mchar(ptr_src, num_src, cvt->encoding_num_from);
				num_src -= len_wchar;
				ptr_src += len_wchar;
			}
		}
		{
			const size_t result = libiconv(cvt->cd, NULL, NULL, &ptr_dst, &num_dst);
			if ((size_t)(-1) != result) {
			} else {
				LOGGER_ERR(l,N_("State error at %1$s %2$zu"), "wrapper_iconv()", len_src);
			}
		}
	}
	dst[ ((0 < num_dst) ? ((size_t)maxlen_dst - num_dst) : ((size_t)maxlen_dst - 1)) ] = '\0';
	return ((size_t)maxlen_dst - num_dst);
}



/** ꤷʸɤʸֹѡˤ
 *@param			enc_name			ʸ
 *@return								ʸֹѡ
 *
 *@author	G-HAL
 *@date		Tue,17 Sep,2013
 */
static enum ENCODING_NUM get_encoding_num(const char enc_name[])
{
	assert (NULL != enc_name);

	for (size_t i = 0; i < number_(encoding_name2num_table); ++i) {
		if (0 == strcasecmp(encoding_name2num_table[i].encoding_name, enc_name)) {
			return encoding_name2num_table[i].encoding_num;
		}
	}
	return ENCODING_MAX;
}


/** ꤷʸɤѴơ֥
 *@param			enc_tbl				ʸ
 *@retval			true				
 *@retval			false				
 *
 *@author	G-HAL
 *@date		Sun,07 Jun,2009 - Wed,10 Jun,2009
 *@date		Tue,16 Jun,2009
 *@date		Wed,17 Jun,2009
 *@date		Mon,16 Sep,2013 - Wed,18 Sep,2013
 */
static bool open_iconv(enum ENCODING_TABLE enc_tbl)
{
	if (ENCODING_TABLE_MAX <= enc_tbl) {
		LOGGER_CRIT(l,N_("Illigal argment at %1$s %2$u"), "open_iconv()", enc_tbl);
		return false;
	}
	{
		struct encoding_converter_table_t* const cvt = &(converter_tbl[enc_tbl]);
		if (cvt->initialized) {
			return true;
		}
		if ((NULL == cvt->encoding_name_from) || ('\0' == cvt->encoding_name_from[0])) {
			LOGGER_CRIT(l,N_("iconv encoding name is empty in %1$u."), enc_tbl);
			return false;
		}
		if ((NULL == cvt->encoding_name_to) || ('\0' == cvt->encoding_name_to[0])) {
			LOGGER_CRIT(l,N_("iconv encoding name is empty in %1$u."), enc_tbl);
			return false;
		}

		cvt->cd = libiconv_open(cvt->encoding_name_to, cvt->encoding_name_from);
		if ((iconv_t)(-1) == cvt->cd) {
			LOGGER_ERR(l,N_("iconv initialization failed in %1$u."), enc_tbl);
			return false;
		}

		{
			const size_t result = libiconv(cvt->cd, NULL, NULL, NULL, NULL);
			if ((size_t)(-1) != result) {
			} else {
				LOGGER_ERR(l,N_("State error at %1$s"), "open_iconv()");
			}
		}

		cvt->GETA[0] = '\0';
		cvt->len_GETA = 0;
		cvt->initialized = true;
	}
	return true;
}


/** ꤷʸɤѴơ֥θ
 *@param			enc_tbl				ʸ
 *@retval			true				
 *@retval			false				
 *
 *@author	G-HAL
 *@date		Mon,16 Sep,2013 - Wed,18 Sep,2013
 */
static bool close_iconv(enum ENCODING_TABLE enc_tbl)
{
	if (ENCODING_TABLE_MAX <= enc_tbl) {
		LOGGER_CRIT(l,N_("Illigal argment at %1$s %2$u"), "close_iconv()", enc_tbl);
		return false;
	}
	bool ret = true;

	struct encoding_converter_table_t* const cvt = &(converter_tbl[enc_tbl]);
	if (cvt->initialized) {
		const int result = libiconv_close(cvt->cd);
		if (0 != result) {
			LOGGER_ERR(l,N_("Failed at %1$s in %2$u %3$s"), "close_iconv()", enc_tbl, strerror(errno));
			ret = false;
		}
		cvt->initialized = false;
	}
	return ret;
}



/** ʸѴν
 *@retval			true				
 *@retval			false				
 *
 *@author	G-HAL
 *@date		Mon,16 Sep,2013 - Wed,18 Sep,2013
 *@date		Mon,30 Sep,2013 - Tue,01 Oct,2013
 */
bool init_iconv(void)
{
	{	/* ѥ᡼ */
		converter_tbl[ENCODING_TABLE_SOURCECODE2INTERNAL].encoding_name_from = args.codeset.sourcecode;
		converter_tbl[ENCODING_TABLE_SOURCECODE2INTERNAL].encoding_name_to   = args.codeset.internal;

		converter_tbl[ENCODING_TABLE_LOCALE2INTERNAL].encoding_name_from = args.codeset.locale;
		converter_tbl[ENCODING_TABLE_LOCALE2INTERNAL].encoding_name_to   = args.codeset.internal;

		converter_tbl[ENCODING_TABLE_INTERNAL2LOCALE].encoding_name_from = args.codeset.internal;
		converter_tbl[ENCODING_TABLE_INTERNAL2LOCALE].encoding_name_to   = args.codeset.locale;

		converter_tbl[ENCODING_TABLE_INTERNAL2CANNAPROTOCOL].encoding_name_from = args.codeset.internal;
		converter_tbl[ENCODING_TABLE_INTERNAL2CANNAPROTOCOL].encoding_name_to   = args.cannaprotocol.codeset;

		converter_tbl[ENCODING_TABLE_CANNAPROTOCOL2INTERNAL].encoding_name_from = args.cannaprotocol.codeset;
		converter_tbl[ENCODING_TABLE_CANNAPROTOCOL2INTERNAL].encoding_name_to   = args.codeset.internal;
	}

	/*  */
	for (enum ENCODING_TABLE i = 0; i < ENCODING_TABLE_MAX; ++i) {
		converter_tbl[i].initialized = false;
		converter_tbl[i].encoding_num_from = get_encoding_num(converter_tbl[i].encoding_name_from);
		converter_tbl[i].encoding_num_to   = get_encoding_num(converter_tbl[i].encoding_name_to);
		const bool ret_oi = open_iconv(i);
		if (!ret_oi) {
			return false;
		}
	}

	{	/* ʸν */
		struct encoding_converter_table_t* const cvt_s2i = &(converter_tbl[ENCODING_TABLE_SOURCECODE2INTERNAL]);
		struct encoding_converter_table_t* const cvt_l2i = &(converter_tbl[ENCODING_TABLE_LOCALE2INTERNAL]);
		struct encoding_converter_table_t* const cvt_i2l = &(converter_tbl[ENCODING_TABLE_INTERNAL2LOCALE]);
		struct encoding_converter_table_t* const cvt_i2c = &(converter_tbl[ENCODING_TABLE_INTERNAL2CANNAPROTOCOL]);
		struct encoding_converter_table_t* const cvt_c2i = &(converter_tbl[ENCODING_TABLE_CANNAPROTOCOL2INTERNAL]);

		cvt_s2i->len_GETA = wrapper_iconv(ENCODING_TABLE_SOURCECODE2INTERNAL, cvt_s2i->GETA, number_(cvt_s2i->GETA), rk_GETA, strlen(rk_GETA));

		strlcpy(cvt_l2i->GETA, cvt_s2i->GETA, number_(cvt_l2i->GETA));
		cvt_l2i->len_GETA = cvt_s2i->len_GETA;

		cvt_i2l->len_GETA = wrapper_iconv(ENCODING_TABLE_INTERNAL2LOCALE, cvt_i2l->GETA, number_(cvt_i2l->GETA), cvt_s2i->GETA, cvt_s2i->len_GETA);

		cvt_i2c->len_GETA = wrapper_iconv(ENCODING_TABLE_INTERNAL2CANNAPROTOCOL, cvt_i2c->GETA, number_(cvt_i2c->GETA), cvt_s2i->GETA, cvt_s2i->len_GETA);

		strlcpy(cvt_c2i->GETA, cvt_s2i->GETA, number_(cvt_c2i->GETA));
		cvt_c2i->len_GETA = cvt_s2i->len_GETA;
	}

	return true;
}


/** ʸѴθ
 *@retval			true				
 *@retval			false				
 *
 *@author	G-HAL
 *@date		Mon,16 Sep,2013
 */
bool fin_iconv(void)
{
	for (enum ENCODING_TABLE i = 0; i < ENCODING_TABLE_MAX; ++i) {
		const bool ret_ci = close_iconv(i);
		if (!ret_ci) {
			return false;
		}
	}

	return true;
}

/* [ End of File ] */
