/**@file
 *@brief	Ͽؿ
 *@date		Mon,26 Aug,2013 - Sat,21 Sep,2013
 *@author	Copyright(C)2005-2013 G-HAL, All rights reserved.
 *
 *@note
 *@verbatim
Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions
are met:

* Redistributions of source code must retain the above copyright
notice, this list of conditions and the following disclaimer.

* Redistributions in binary form must reproduce the above copyright
notice, this list of conditions and the following disclaimer in the
documentation and/or other materials provided with the distribution.

* Neither the name of the copyright holders nor the names of its contributors
may be used to endorse or promote products derived from this software
without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
"AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE COPYRIGHT OWNER OR
CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *@endverbatim
 */

#if defined(HAVE_CONFIG_H)
# include "config.h"
#endif

#if defined(HAVE_STDLIB_H)
# include <stdlib.h>
#endif
#if defined(HAVE_STDIO_H)
# include <stdio.h>
#endif
#if defined(HAVE_STDBOOL_H)
# include <stdbool.h>
#endif

#if defined(HAVE_ASSERT_H)
# include <assert.h>
#endif

#if !defined(__STDC_LIMIT_MACROS)
# define __STDC_LIMIT_MACROS
#endif
#if !defined(__STDC_CONSTANT_MACROS)
# define __STDC_CONSTANT_MACROS
#endif
#if defined(HAVE_STDINT_H)
# include <stdint.h>
#endif
#if !defined(__STDC_FORMAT_MACROS)
# define __STDC_FORMAT_MACROS
#endif
#if defined(HAVE_INTTYPES_H)
# include <inttypes.h>
#endif

#if defined(HAVE_STRING_H)
# include <string.h>
#endif
#if defined(HAVE_SYS_TYPES_H)
# include <sys/types.h>
#endif
#if defined(HAVE_SYS_STAT_H)
# include <sys/stat.h>
#endif
#if defined(HAVE_STDARG_H)
# include <stdarg.h>
#endif
#if defined(HAVE_LIMITS_H)
# include <limits.h>
#endif
#if defined(HAVE_SYSLOG_H)
# include <syslog.h>
#endif
#if defined(TIME_WITH_SYS_TIME)
#if defined(HAVE_SYS_TIME_H)
# include <sys/time.h>
#endif
#if defined(HAVE_TIME_H)
# include <time.h>
#endif
#endif
#if defined(HAVE_PTHREAD_H)
# include <pthread.h>
#endif

#include "intl.h"
#include "logger.h"



/** Υץ饤ƥɽʸ
 */
const char* const PriorityStr[LOGGER_LOG_MAX_PRIORITY] = {
	"EMERG",
	"ALERT",
	"CRIT",
	"ERROR",
	"WARN",
	"NOTICE",
	"INFO",
	"DEBUG",
	"TRACE",
};



struct logger* l = NULL;



/** ϼΡĹ
 *@param[in]		log			
 *@param[in]		priority	
 *@param[in]		msg			printfϥå
 *
 *@author	G-HAL
 *@date		Mon,26 Aug,2013 - Thu,29 Aug,2013
 */
#define	WRITE(log, priority, msg)	\
{ \
	assert (NULL != log); \
	assert (NULL != msg); \
	struct timeval tp; \
	time_t current; \
	struct tm local; \
	gettimeofday(&tp, NULL); \
	current = tp.tv_sec; \
	localtime_r(&current, &local); \
	pthread_mutex_lock(&(log->mutex)); \
	fprintf(log->fp, "%04d-%02d-%02d %02d:%02d:%02d(%010"PRId64".%06"PRId32") %s " \
			, \
			(int)(local.tm_year + 1900), \
			(int)(local.tm_mon + 1), \
			(int)(local.tm_mday), \
			(int)(local.tm_hour), \
			(int)(local.tm_min), \
			(int)(local.tm_sec), \
			(int64_t)(tp.tv_sec), \
			(int32_t)(tp.tv_usec), \
			(0 <= priority && priority <= LOGGER_LOG_MAX_PRIORITY) ? PriorityStr[priority] : "" \
			); \
	{ \
		va_list	ap; \
		va_start(ap, msg); \
		vfprintf(log->fp, _(msg),ap); \
		va_end(ap); \
	} \
	fprintf(log->fp, "\n"); \
	pthread_mutex_unlock(&(log->mutex)); \
}



/** ϡĹǡץ饤ƥåդ
 *@param[in]		log			
 *@param			priority	
 *@param[in]		msg			ϥå
 *@param[in]		...			ϥå printf
 *
 *@author	G-HAL
 *@date		Mon,26 Aug,2013 - Thu,29 Aug,2013
 */
void logger_writeWpriority(struct logger* const log, int priority, const char* msg, ...)
{
	assert (NULL != log);
	assert (NULL != msg);
	if (log->output_priority < priority) {
		return;
	}
	WRITE(log, priority, msg);
	return;
}



/** ϼΡĹǡ
 *@param[in]		log			
 *@param[in]		name		ϥåե̾
 *@param			line		ϥåֹ
 *@param[in]		func		ϥåؿ̾
 *@param			priority	
 *@param[in]		msg			ϥå
 *@param[in]		...			ϥå printf
 *
 *@author	G-HAL
 *@date		Mon,26 Aug,2013 - Thu,29 Aug,2013
 */
void logger_writeWtag(struct logger* const log, const char* name, int line, const char* func, int priority, const char* msg, ...)
{
	assert (NULL != log);
	assert (NULL != name);
	assert (0 < line);
	assert (NULL != func);
	assert (NULL != msg);
	if (log->output_priority < priority) {
		return;
	}

	{
		struct timeval tp;
		time_t current;
		struct tm local;
		gettimeofday(&tp, NULL);
		current = tp.tv_sec;
		localtime_r(&current, &local);

		pthread_mutex_lock(&(log->mutex));
		fprintf(log->fp,
				"%04d-%02d-%02d %02d:%02d:%02d(%010"PRId64".%06"PRId32") %s "
				"%s:%d:%s; ",
				(int)(local.tm_year + 1900),
				(int)(local.tm_mon + 1),
				(int)(local.tm_mday),
				(int)(local.tm_hour),
				(int)(local.tm_min),
				(int)(local.tm_sec),
				(int64_t)(tp.tv_sec),
				(int32_t)(tp.tv_usec),
				(0 <= priority && priority <= LOGGER_LOG_MAX_PRIORITY) ? PriorityStr[priority] : "",
				name, line, func);
		{
			va_list ap;
			va_start(ap, msg);
			vfprintf(log->fp, _(msg),ap);
			va_end(ap);
		}
		fprintf(log->fp, "\n");
		pthread_mutex_unlock(&(log->mutex));
	}
	return;
}



/** Ϥν
 *
 *@note
 *		ϥǥ쥯ȥ꤬ʤк
 *
 *@attention
 *		δؿ static
 *		δؿ󥹥åɥա
 *
 *@author	G-HAL
 *@date		Mon,26 Aug,2013 - Thu,29 Aug,2013
 *@date		Tue,03 Sep,2013
 *@date		Tue,10 Sep,2013
 */
void init_logger(void)
{
	if (NULL == l) {
		l = malloc(sizeof(*l));
		l->fp = NULL;
		l->output_priority = LOG_INFO;
		l->logdir = NULL;
		l->logfile = NULL;

		const int mi = pthread_mutex_init(&(l->mutex), NULL);
		if (mi) {
			fprintf(stderr,"init_logger(): pthread_*_init() Failed:%d", mi);
			abort();
		}
	}

	/* ϥ٥μ */
	const char* const env_str_priority = getenv(LOGGER_LOGPRIORITY);
	if (NULL != env_str_priority) {
		l->output_priority = (int)strtol(env_str_priority, NULL, 10);
	}

	/* μ */
	const char* const env_logdir = getenv(LOGGER_LOGHOME);
	if (!env_logdir) {
		l->fp = stderr;
		LOGGER_DEBUG(l,N_("No specification LOGHOME. Log will output to stderr. At %1$s"), "init_logger()");
		return;
	}
	l->logdir = strdup(env_logdir);
	if (!l->logdir) {
		LOGGER_CRIT(l,N_("Out of memory at %1$s"), "init_logger()");
		abort();
	}
	const char* const env_logfile = getenv(LOGGER_LOGFILE);
	if (!env_logfile) {
		l->fp = stderr;
		LOGGER_DEBUG(l,N_("No specification LOGFILE. Log will output to stderr. At %1$s"), "init_logger()");
		return;
	}
	l->logfile = strdup(env_logfile);
	if (!l->logfile) {
		LOGGER_CRIT(l,N_("Out of memory at %1$s"), "init_logger()");
		abort();
	}

	/* ϥǥ쥯ȥ꤬¸ߤ뤫å */
	struct stat file_stat;
	const int ret_stat = stat(l->logdir, &file_stat);
	if (0 != ret_stat) {
		/* ϥǥ쥯ȥ̵꤬ΤǺ */
		const int ret_mkdir = mkdir(l->logdir, (S_IWGRP|S_IWOTH));
		if (0 != ret_mkdir) {
			l->fp = stderr;
			LOGGER_ERROR(l,N_("Can't access logdir '%2$s'. Log will output to stderr. At %1$s"), "init_logger()", l->logdir);
			return;
		}
	}

	/* ե򥪡ץ */
	snprintf(l->logname, number_(l->logname),
			 "%s/%s",
			 l->logdir, l->logfile);
	l->fp = fopen(l->logname, "a");
	if (NULL == l->fp) {
		l->fp = stderr;
		LOGGER_ERROR(l,N_("Can't open logfile '%2$s'. Log will output to stderr. At %1$s"), "init_logger()", l->logname);
		return;
	}

	return;
}



/** Ϥθ
 *
 *@attention
 *		δؿ static
 *
 *@author	G-HAL
 *@date		Mon,26 Aug,2013 - Thu,29 Aug,2013
 *@date		Tue,03 Sep,2013
 *@date		Tue,10 Sep,2013
 */
void fin_logger(void)
{
	if (l) {
		const int md = pthread_mutex_destroy(&(l->mutex));
		if (md) {
			fprintf(stderr,"fin_logger(): pthread_*_destroy() Failed:%d", md);
		}

		fflush(l->fp);
		free(l->logfile);
		free(l->logdir);
		free(l);
		l = NULL;
	}
	return;
}

/* [ End of File ] */
