/**@file
 *@brief	Ͽؿ
 *@date		Mon,26 Aug,2013 - Tue,10 Sep,2013
 *@author	Copyright(C)2005-2013 G-HAL, All rights reserved.
 *
 *@note
 *@verbatim
Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions
are met:

* Redistributions of source code must retain the above copyright
notice, this list of conditions and the following disclaimer.

* Redistributions in binary form must reproduce the above copyright
notice, this list of conditions and the following disclaimer in the
documentation and/or other materials provided with the distribution.

* Neither the name of the copyright holders nor the names of its contributors
may be used to endorse or promote products derived from this software
without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
"AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE COPYRIGHT OWNER OR
CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *@endverbatim
 */
#ifndef logger_h_included_
#define logger_h_included_

#if defined(HAVE_CONFIG_H)
# include "config.h"
#endif

#if defined(HAVE_STDLIB_H)
# include <stdlib.h>
#endif
#if defined(HAVE_STDIO_H)
# include <stdio.h>
#endif

#if defined(HAVE_STDARG_H)
# include <stdarg.h>
#endif
#if defined(HAVE_PTHREAD_H)
# include <pthread.h>
#endif
#if defined(HAVE_LIMITS_H)
# include <limits.h>
#endif

#if defined(HAVE_SYSLOG_H)
# include <syslog.h>
#else
# define	LOG_EMERG	0	///< system is unusableg
# define	LOG_ALERT	1	///< action must be taken immediately
# define	LOG_CRIT	2	///< critical conditionsg
# define	LOG_ERR		3	///< error conditionsg
# define	LOG_WARNING	4	///< warning conditionsg
# define	LOG_NOTICE	5	///< normal but significant conditiong
# define	LOG_INFO	6	///< informationalg
# define	LOG_DEBUG	7	///< debug-level messagesg
#endif
#define		LOG_TRACE	8	///< trace messagesg

#include "environment_check.h"



#ifdef __cplusplus
extern "C" {
#endif



#if !defined(LOGGER_LOGHOME)
# define	LOGGER_LOGHOME				"LOGHOME"	/**< ǥ쥯ȥꤹĶѿ̾ */
#endif
#if !defined(LOGGER_LOGFILE)
# define	LOGGER_LOGFILE				"LOGFILE"	/**< ե̾ꤹĶѿ̾ */
#endif
#if !defined(LOGGER_LOGPRIORITY)
# define	LOGGER_LOGPRIORITY			"LOGPRIORITY"	/**< Ϥ٤ꤹĶѿ̾ */
#endif
#define	LOGGER_LOGBUF_NUM			512			/**< ΥåХåե */
#define	LOGGER_LOG_MAX_PRIORITY		9			/**< ٤ʿʼ֤ס */

/** 
 */
struct logger {
	FILE* fp;								/**< եΥǥץ */
	int output_priority;					/**< ˽Ϥ롢ͥ */
	char* logdir;							/**< ǥ쥯ȥ̾ */
	char* logfile;							/**< ե̾ */
	char logname[PATH_MAX];					/**< եѥե̾ */
	pthread_mutex_t mutex;					/**< Ʊ϶ػ */
};



extern struct logger* l;

extern void init_logger(void);
extern void fin_logger(void);
extern void logger_writeWpriority(struct logger* const __restrict log, int priority, const char* __restrict msg, ...) __printflike(3,4);
extern void logger_writeWtag(struct logger* const __restrict log, const char* name, int line, const char* func, int priority, const char* __restrict msg, ...) __printflike(6,7);



#if (199901L <= (__STDC_VERSION__ +0L))
# define	logwriteTAG(l,f,...)	logger_writeWtag(l,__FILE__,__LINE__,__FUNCTION__,f ,__VA_ARGS__ )
#elif (2 <= (__GNUC__ +0))
# define	logwriteTAG(l,f,arg...)	logger_writeWtag(l,__FILE__,__LINE__,__func__,f ,##arg )
#else
# define	logwriteTAG(l,f,arg...)	logger_writeWtag(l,__FILE__,__LINE__,"",f ,##arg )
#endif

#if defined(ENABLE_LOGTAG)
# if (199901L <= (__STDC_VERSION__ +0L))
#  define	logwriteT(l,f,...)			logger_writeWtag(l,__FILE__,__LINE__,__FUNCTION__,f ,__VA_ARGS__ )
# elif (2 <= (__GNUC__ +0))
#  define	logwriteT(l,f,arg...)		logger_writeWtag(l,__FILE__,__LINE__,__func__,f ,##arg )
# else
#  define	logwriteT(l,f,arg...)		logger_writeWtag(l,__FILE__,__LINE__,"",f ,##arg )
# endif
#else
# if (199901L <= (__STDC_VERSION__ +0L))
#  define	logwriteT(l,f,...)			logger_writeWpriority(l,f ,__VA_ARGS__ )
# elif (2 <= (__GNUC__ +0))
#  define	logwriteT(l,f,arg...)		logger_writeWpriority(l,f ,##arg )
# else
#  define	logwriteT(l,f,arg...)		logger_writeWpriority(l,f ,##arg )
# endif
#endif

#if !defined(LOGGER_DISABLE_TRACE) && (3)<=(DEBUG+0)
# if (199901L <= (__STDC_VERSION__ +0L))
#  define	LOGGER_TRACE3(l,...)	logwriteTAG(l,LOG_TRACE ,__VA_ARGS__ )
# else
#  define	LOGGER_TRACE3(l,arg...)	logwriteTAG(l,LOG_TRACE ,##arg )
# endif
#else
# if (199901L <= (__STDC_VERSION__ +0L))
#  define	LOGGER_TRACE3(l,...)
# else
#  define	LOGGER_TRACE3(l,arg...)
# endif
#endif

#if !defined(LOGGER_DISABLE_TRACE) && (2)<=(DEBUG+0)
# if (199901L <= (__STDC_VERSION__ +0L))
#  define	LOGGER_TRACE2(l,...)	logwriteTAG(l,LOG_TRACE ,__VA_ARGS__ )
# else
#  define	LOGGER_TRACE2(l,arg...)	logwriteTAG(l,LOG_TRACE ,##arg )
# endif
#else
# if (199901L <= (__STDC_VERSION__ +0L))
#  define	LOGGER_TRACE2(l,...)
# else
#  define	LOGGER_TRACE2(l,arg...)
# endif
#endif

#if !defined(LOGGER_DISABLE_TRACE) && (1)<=(DEBUG+0)
# if (199901L <= (__STDC_VERSION__ +0L))
#  define	LOGGER_TRACE1(l,...)	logwriteTAG(l,LOG_TRACE ,__VA_ARGS__ )
# else
#  define	LOGGER_TRACE1(l,arg...)	logwriteTAG(l,LOG_TRACE ,##arg )
# endif
#else
# if (199901L <= (__STDC_VERSION__ +0L))
#  define	LOGGER_TRACE1(l,...)
# else
#  define	LOGGER_TRACE1(l,arg...)
# endif
#endif

#if !defined(LOGGER_DISABLE_TRACE) && defined(DEBUG)
# if (199901L <= (__STDC_VERSION__ +0L))
#  define	LOGGER_TRACE0(l,...)	logwriteTAG(l,LOG_TRACE ,__VA_ARGS__ )
# else
#  define	LOGGER_TRACE0(l,arg...)	logwriteTAG(l,LOG_TRACE ,##arg )
# endif
#else
# if (199901L <= (__STDC_VERSION__ +0L))
#  define	LOGGER_TRACE0(l,...)
# else
#  define	LOGGER_TRACE0(l,arg...)
# endif
#endif

#if !defined(LOGGER_DISABLE_TRACE)
# if (199901L <= (__STDC_VERSION__ +0L))
#  define	LOGGER_TRACE(l,...)		logwriteTAG(l,LOG_TRACE ,__VA_ARGS__ )
# else
#  define	LOGGER_TRACE(l,arg...)	logwriteTAG(l,LOG_TRACE ,##arg )
# endif
#else
# if (199901L <= (__STDC_VERSION__ +0L))
#  define	LOGGER_TRACE(l,...)
# else
#  define	LOGGER_TRACE(l,arg...)
# endif
#endif

#if !defined(LOGGER_DISABLE_DEBUG)
# if (199901L <= (__STDC_VERSION__ +0L))
#  define	LOGGER_DEBUG(l,...)			logwriteT(l,LOG_DEBUG ,__VA_ARGS__ )
# else
#  define	LOGGER_DEBUG(l,arg...)		logwriteT(l,LOG_DEBUG ,##arg )
# endif
#else
# if (199901L <= (__STDC_VERSION__ +0L))
#  define	LOGGER_DEBUG(l,...)
# else
#  define	LOGGER_DEBUG(l,arg...)
# endif
#endif

#if !defined(LOGGER_DISABLE_INFO)
# if (199901L <= (__STDC_VERSION__ +0L))
#  define	LOGGER_INFO(l,...)			logwriteT(l,LOG_INFO ,__VA_ARGS__ )
# else
#  define	LOGGER_INFO(l,arg...)		logwriteT(l,LOG_INFO ,##arg )
# endif
#else
# if (199901L <= (__STDC_VERSION__ +0L))
#  define	LOGGER_INFO(l,...)
# else
#  define	LOGGER_INFO(l,arg...)
# endif
#endif

#if !defined(LOGGER_DISABLE_NOTICE)
# if (199901L <= (__STDC_VERSION__ +0L))
#  define	LOGGER_NOTICE(l,...)		logwriteT(l,LOG_NOTICE ,__VA_ARGS__ )
# else
#  define	LOGGER_NOTICE(l,arg...)		logwriteT(l,LOG_NOTICE ,##arg )
# endif
#else
# if (199901L <= (__STDC_VERSION__ +0L))
#  define	LOGGER_NOTICE(l,...)
# else
#  define	LOGGER_NOTICE(l,arg...)
# endif
#endif

#if !defined(LOGGER_DISABLE_WARNING)
# if (199901L <= (__STDC_VERSION__ +0L))
#  define	LOGGER_WARN(l,...)		logwriteT(l,LOG_WARNING ,__VA_ARGS__ )
#  define	LOGGER_WARNING(l,...)		logwriteT(l,LOG_WARNING ,__VA_ARGS__ )
# else
#  define	LOGGER_WARN(l,arg...)	logwriteT(l,LOG_WARNING ,##arg )
#  define	LOGGER_WARNING(l,arg...)	logwriteT(l,LOG_WARNING ,##arg )
# endif
#else
# if (199901L <= (__STDC_VERSION__ +0L))
#  define	LOGGER_WARN(l,...)
#  define	LOGGER_WARNING(l,...)
# else
#  define	LOGGER_WARN(l,arg...)
#  define	LOGGER_WARNING(l,arg...)
# endif
#endif

#if !defined(LOGGER_DISABLE_ERR)
# if (199901L <= (__STDC_VERSION__ +0L))
#  define	LOGGER_ERR(l,...)			logwriteT(l,LOG_ERR ,__VA_ARGS__ )
#  define	LOGGER_ERROR(l,...)			logwriteT(l,LOG_ERR ,__VA_ARGS__ )
# else
#  define	LOGGER_ERR(l,arg...)		logwriteT(l,LOG_ERR ,##arg )
#  define	LOGGER_ERROR(l,arg...)		logwriteT(l,LOG_ERR ,##arg )
# endif
#else
# if (199901L <= (__STDC_VERSION__ +0L))
#  define	LOGGER_ERR(l,...)
#  define	LOGGER_ERROR(l,...)
# else
#  define	LOGGER_ERR(l,arg...)
#  define	LOGGER_ERROR(l,arg...)
# endif
#endif

#if !defined(LOGGER_DISABLE_CRIT)
# if (199901L <= (__STDC_VERSION__ +0L))
#  define	LOGGER_CRIT(l,...)			logwriteT(l,LOG_CRIT ,__VA_ARGS__ )
# else
#  define	LOGGER_CRIT(l,arg...)		logwriteT(l,LOG_CRIT ,##arg )
# endif
#else
# if (199901L <= (__STDC_VERSION__ +0L))
#  define	LOGGER_CRIT(l,...)
# else
#  define	LOGGER_CRIT(l,arg...)
# endif
#endif

#if !defined(LOGGER_DISABLE_ALERT)
# if (199901L <= (__STDC_VERSION__ +0L))
#  define	LOGGER_ALERT(l,...)			logwriteT(l,LOG_ALERT ,__VA_ARGS__ )
# else
#  define	LOGGER_ALERT(l,arg...)		logwriteT(l,LOG_ALERT ,##arg )
# endif
#else
# if (199901L <= (__STDC_VERSION__ +0L))
#  define	LOGGER_ALERT(l,...)
# else
#  define	LOGGER_ALERT(l,arg...)
# endif
#endif

#if !defined(LOGGER_DISABLE_EMERG)
# if (199901L <= (__STDC_VERSION__ +0L))
#  define	LOGGER_EMERG(l,...)			logwriteT(l,LOG_EMERG ,__VA_ARGS__ )
# else
#  define	LOGGER_EMERG(l,arg...)		logwriteT(l,LOG_EMERG ,##arg )
# endif
#else
# if (199901L <= (__STDC_VERSION__ +0L))
#  define	LOGGER_EMERG(l,...)
# else
#  define	LOGGER_EMERG(l,arg...)
# endif
#endif



#ifdef __cplusplus
}
#endif

#endif /* logger_h_included_ */
/* [ End of File ] */
