/**@file
 *@brief	anthy-agentץȥʹؿRKѴơ֥
 *@date		Wed,28 Aug,2013 - Wed,18 Sep,2013
 *@author	Copyright(C)2013 G-HAL, All rights reserved.
 *
 *@note
 *@verbatim
Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions
are met:

* Redistributions of source code must retain the above copyright
notice, this list of conditions and the following disclaimer.

* Redistributions in binary form must reproduce the above copyright
notice, this list of conditions and the following disclaimer in the
documentation and/or other materials provided with the distribution.

* Neither the name of the copyright holders nor the names of its contributors
may be used to endorse or promote products derived from this software
without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
"AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE COPYRIGHT OWNER OR
CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *@endverbatim
 */

#if defined(HAVE_CONFIG_H)
# include "config.h"
#endif

#include "parseline_rk.h"



const char rk_GETA[] = "";


static const struct rk_map_source_t rk_map_hiragana[] = {
	{	"1",	"",	"\0"	},
	{	"2",	"",	"\0"	},
	{	"3",	"",	"\0"	},
	{	"4",	"",	"\0"	},
	{	"5",	"",	"\0"	},
	{	"6",	"",	"\0"	},
	{	"7",	"",	"\0"	},
	{	"8",	"",	"\0"	},
	{	"9",	"",	"\0"	},
	{	"0",	"",	"\0"	},

	{	"a",	"",	"\0"	},
	{	"i",	"",	"\0"	},
	{	"u",	"",	"\0"	},
	{	"e",	"",	"\0"	},
	{	"o",	"",	"\0"	},

	{	"xa",	"",	"\0"	},
	{	"xi",	"",	"\0"	},
	{	"xu",	"",	"\0"	},
	{	"xe",	"",	"\0"	},
	{	"xo",	"",	"\0"	},

	{	"ka",	"",	"\0"	},
	{	"ki",	"",	"\0"	},
	{	"ku",	"",	"\0"	},
	{	"ke",	"",	"\0"	},
	{	"ko",	"",	"\0"	},

	{	"kya",	"",	"\0"	},
	{	"kyi",	"",	"\0"	},
	{	"kyu",	"",	"\0"	},
	{	"kye",	"",	"\0"	},
	{	"kyo",	"",	"\0"	},

	{	"kwa",	"",	"\0"	},
	{	"kwi",	"",	"\0"	},
	{	"kwu",	""  ,	"\0"	},
	{	"kwe",	"",	"\0"	},
	{	"kwo",	"",	"\0"	},

	{	"qya",	"",	"\0"	},
	{	"qyi",	"",	"\0"	},
	{	"qyu",	"",	"\0"	},
	{	"qye",	"",	"\0"	},
	{	"qyo",	"",	"\0"	},

	{	"sa",	"",	"\0"	},
	{	"si",	"",	"\0"	},
	{	"su",	"",	"\0"	},
	{	"se",	"",	"\0"	},
	{	"so",	"",	"\0"	},

	{	"ci",	"",	"\0"	},
	{	"ce",	"",	"\0"	},

	{	"sya",	"",	"\0"	},
	{	"syi",	"",	"\0"	},
	{	"syu",	"",	"\0"	},
	{	"sye",	"",	"\0"	},
	{	"syo",	"",	"\0"	},

	{	"sha",	"",	"\0"	},
	{	"shi",	""  ,	"\0"	},
	{	"shu",	"",	"\0"	},
	{	"she",	"",	"\0"	},
	{	"sho",	"",	"\0"	},

	{	"ta",	"",	"\0"	},
	{	"ti",	"",	"\0"	},
	{	"tu",	"",	"\0"	},
	{	"te",	"",	"\0"	},
	{	"to",	"",	"\0"	},

	{	"ltu",	"",	"\0"	},
	{	"ltsu",	"",	"\0"	},

	{	"tya",	"",	"\0"	},
	{	"tyi",	"",	"\0"	},
	{	"tyu",	"",	"\0"	},
	{	"tye",	"",	"\0"	},
	{	"tyo",	"",	"\0"	},

	{	"cya",	"",	"\0"	},
	{	"cyi",	"",	"\0"	},
	{	"cyu",	"",	"\0"	},
	{	"cye",	"",	"\0"	},
	{	"cyo",	"",	"\0"	},

	{	"cha",	"",	"\0"	},
	{	"chi",	""  ,	"\0"	},
	{	"chu",	"",	"\0"	},
	{	"che",	"",	"\0"	},
	{	"cho",	"",	"\0"	},

	{	"tsa",	"Ĥ",	"\0"	},
	{	"tsi",	"Ĥ",	"\0"	},
	{	"tsu",	""  ,	"\0"	},
	{	"tse",	"Ĥ",	"\0"	},
	{	"tso",	"Ĥ",	"\0"	},

	{	"tha",	"Ƥ",	"\0"	},
	{	"thi",	"Ƥ",	"\0"	},
	{	"thu",	"Ƥ",	"\0"	},
	{	"the",	"Ƥ",	"\0"	},
	{	"tho",	"Ƥ",	"\0"	},

	{	"twu",	"Ȥ",	"\0"	},

	{	"na",	"",	"\0"	},
	{	"ni",	"",	"\0"	},
	{	"nu",	"",	"\0"	},
	{	"ne",	"",	"\0"	},
	{	"no",	"",	"\0"	},

	{	"nya",	"ˤ",	"\0"	},
	{	"nyi",	"ˤ",	"\0"	},
	{	"nyu",	"ˤ",	"\0"	},
	{	"nye",	"ˤ",	"\0"	},
	{	"nyo",	"ˤ",	"\0"	},

	{	"ha",	"",	"\0"	},
	{	"hi",	"",	"\0"	},
	{	"hu",	"",	"\0"	},
	{	"he",	"",	"\0"	},
	{	"ho",	"",	"\0"	},

	{	"hya",	"Ҥ",	"\0"	},
	{	"hyi",	"Ҥ",	"\0"	},
	{	"hyu",	"Ҥ",	"\0"	},
	{	"hye",	"Ҥ",	"\0"	},
	{	"hyo",	"Ҥ",	"\0"	},

	{	"fa",	"դ",	"\0"	},
	{	"fi",	"դ",	"\0"	},
	{	"fu",	""  ,	"\0"	},
	{	"fe",	"դ",	"\0"	},
	{	"fo",	"դ",	"\0"	},

	{	"fya",	"դ",	"\0"	},
	{	"fyi",	"դ",	"\0"	},
	{	"fyu",	"դ",	"\0"	},
	{	"fye",	"դ",	"\0"	},
	{	"fyo",	"դ",	"\0"	},

	{	"ma",	"",	"\0"	},
	{	"mi",	"",	"\0"	},
	{	"mu",	"",	"\0"	},
	{	"me",	"",	"\0"	},
	{	"mo",	"",	"\0"	},

	{	"mya",	"ߤ",	"\0"	},
	{	"myi",	"ߤ",	"\0"	},
	{	"myu",	"ߤ",	"\0"	},
	{	"mye",	"ߤ",	"\0"	},
	{	"myo",	"ߤ",	"\0"	},

	{	"ya",	"",	"\0"	},
	{	"yi",	"",	"\0"	},
	{	"yu",	"",	"\0"	},
	{	"ye",	"",	"\0"	},
	{	"yo",	"",	"\0"	},

	{	"lya",	"",	"\0"	},
	{	"lye",	"ꤣ",	"\0"	},
	{	"lyi",	"",	"\0"	},
	{	"lyo",	"ꤧ",	"\0"	},
	{	"lyu",	"",	"\0"	},

	{	"xya",	"",	"\0"	},
	{	"xyi",	"",	"\0"	},
	{	"xyu",	"",	"\0"	},
	{	"xye",	"",	"\0"	},
	{	"xyo",	"",	"\0"	},

	{	"ra",	"",	"\0"	},
	{	"ri",	"",	"\0"	},
	{	"ru",	"",	"\0"	},
	{	"re",	"",	"\0"	},
	{	"ro",	"",	"\0"	},

	{	"la",	"",	"\0"	},
	{	"li",	"",	"\0"	},
	{	"lu",	"",	"\0"	},
	{	"le",	"",	"\0"	},
	{	"lo",	"",	"\0"	},

	{	"rya",	"",	"\0"	},
	{	"ryi",	"ꤣ",	"\0"	},
	{	"ryu",	"",	"\0"	},
	{	"rye",	"ꤧ",	"\0"	},
	{	"ryo",	"",	"\0"	},

	{	"wa",	"",	"\0"	},
	{	"wi",	"",	"\0"	},
	{	"wu",	"",	"\0"	},
	{	"we",	"",	"\0"	},
	{	"wo",	"",	"\0"	},

	{	"wha",	"",	"\0"	},
	{	"whi",	"",	"\0"	},
	{	"whu",	""  ,	"\0"	},
	{	"whe",	"",	"\0"	},
	{	"who",	"",	"\0"	},

	{	"xwa",	"",	"\0"	},
	{	"xtu",	"",	"\0"	},
	{	"xtsu",	"",	"\0"	},

	{	"WA",	"",	"\0"	},
	{	"TU",	"",	"\0"	},
	{	"TSU",	"",	"\0"	},
	{	"YA",	"",	"\0"	},
	{	"YU",	"",	"\0"	},
	{	"YO",	"",	"\0"	},
	{	"ll",	"",	"l"		},

	{	"nn",	"",	"\0"	},
	{	"mn",	"",	"\0"	},

	{	"ga",	"",	"\0"	},
	{	"gi",	"",	"\0"	},
	{	"gu",	"",	"\0"	},
	{	"ge",	"",	"\0"	},
	{	"go",	"",	"\0"	},

	{	"gya",	"",	"\0"	},
	{	"gyi",	"",	"\0"	},
	{	"gyu",	"",	"\0"	},
	{	"gye",	"",	"\0"	},
	{	"gyo",	"",	"\0"	},

	{	"gwa",	"",	"\0"	},
	{	"gwi",	"",	"\0"	},
	{	"gwu",	"",	"\0"	},
	{	"gwe",	"",	"\0"	},
	{	"gwo",	"",	"\0"	},

	{	"za",	"",	"\0"	},
	{	"zi",	"",	"\0"	},
	{	"zu",	"",	"\0"	},
	{	"ze",	"",	"\0"	},
	{	"zo",	"",	"\0"	},

	{	"jya",	"",	"\0"	},
	{	"jyi",	"",	"\0"	},
	{	"jyu",	"",	"\0"	},
	{	"jye",	"",	"\0"	},
	{	"jyo",	"",	"\0"	},

	{	"zya",	"",	"\0"	},
	{	"zyi",	"",	"\0"	},
	{	"zyu",	"",	"\0"	},
	{	"zye",	"",	"\0"	},
	{	"zyo",	"",	"\0"	},

	{	"ja",	"",	"\0"	},
	{	"ji",	""  ,	"\0"	},
	{	"ju",	"",	"\0"	},
	{	"je",	"",	"\0"	},
	{	"jo",	"",	"\0"	},

	{	"da",	"",	"\0"	},
	{	"di",	"",	"\0"	},
	{	"du",	"",	"\0"	},
	{	"de",	"",	"\0"	},
	{	"do",	"",	"\0"	},

	{	"dya",	"¤",	"\0"	},
	{	"dyi",	"¤",	"\0"	},
	{	"dyu",	"¤",	"\0"	},
	{	"dye",	"¤",	"\0"	},
	{	"dyo",	"¤",	"\0"	},

	{	"dha",	"Ǥ",	"\0"	},
	{	"dhi",	"Ǥ",	"\0"	},
	{	"dhu",	"Ǥ",	"\0"	},
	{	"dhe",	"Ǥ",	"\0"	},
	{	"dho",	"Ǥ",	"\0"	},

	{	"dwu",	"ɤ",	"\0"	},

	{	"ba",	"",	"\0"	},
	{	"bi",	"",	"\0"	},
	{	"bu",	"",	"\0"	},
	{	"be",	"",	"\0"	},
	{	"bo",	"",	"\0"	},

	{	"bya",	"Ӥ",	"\0"	},
	{	"byi",	"Ӥ",	"\0"	},
	{	"byu",	"Ӥ",	"\0"	},
	{	"bye",	"Ӥ",	"\0"	},
	{	"byo",	"Ӥ",	"\0"	},

	{	"pa",	"",	"\0"	},
	{	"pi",	"",	"\0"	},
	{	"pu",	"",	"\0"	},
	{	"pe",	"",	"\0"	},
	{	"po",	"",	"\0"	},

	{	"pya",	"Ԥ",	"\0"	},
	{	"pyi",	"Ԥ",	"\0"	},
	{	"pyu",	"Ԥ",	"\0"	},
	{	"pye",	"Ԥ",	"\0"	},
	{	"pyo",	"Ԥ",	"\0"	},

	{	"ca",	"",	"\0"	},
	{	"cu",	"",	"\0"	},
	{	"co",	"",	"\0"	},

	{	"xka",	"",	"\0"	},
	{	"xke",	"",	"\0"	},

	{	"va",	"",	"\0"	},
	{	"vi",	"",	"\0"	},
	{	"vu",	""  ,	"\0"	},
	{	"ve",	"",	"\0"	},
	{	"vo",	"",	"\0"	},

	{	"vya",	"",	"\0"	},
	{	"vyi",	"",	"\0"	},
	{	"vyu",	"",	"\0"	},
	{	"vye",	"",	"\0"	},
	{	"vyo",	"",	"\0"	},

	{	"kk",	"",	"k"	},
	{	"ss",	"",	"s"	},
	{	"tt",	"",	"t"	},
	{	"hh",	"",	"h"	},
	{	"mm",	"",	"m"	},
	{	"yy",	"",	"y"	},
	{	"rr",	"",	"r"	},
	{	"ww",	"",	"w"	},
	{	"gg",	"",	"g"	},
	{	"zz",	"",	"z"	},
	{	"dd",	"",	"d"	},
	{	"bb",	"",	"b"	},
	{	"pp",	"",	"p"	},
	{	"cc",	"",	"c"	},
	{	"ff",	"",	"f"	},
	{	"jj",	"",	"j"	},
	{	"qq",	"",	"q"	},
	{	"vv",	"",	"v"	},
	{	"tch",	"",	"ch"	},

	{	"A",	"",	"\0"	},
	{	"I",	"",	"\0"	},
	{	"U",	"",	"\0"	},
	{	"E",	"",	"\0"	},
	{	"O",	"",	"\0"	},

	{	"Xa",	"",	"\0"	},
	{	"Xi",	"",	"\0"	},
	{	"Xu",	"",	"\0"	},
	{	"Xe",	"",	"\0"	},
	{	"Xo",	"",	"\0"	},

	{	"Ka",	"",	"\0"	},
	{	"Ki",	"",	"\0"	},
	{	"Ku",	"",	"\0"	},
	{	"Ke",	"",	"\0"	},
	{	"Ko",	"",	"\0"	},

	{	"Kya",	"",	"\0"	},
	{	"Kyi",	"",	"\0"	},
	{	"Kyu",	"",	"\0"	},
	{	"Kye",	"",	"\0"	},
	{	"Kyo",	"",	"\0"	},

	{	"Kwa",	"",	"\0"	},
	{	"Kwi",	"",	"\0"	},
	{	"Kwu",	""  ,	"\0"	},
	{	"Kwe",	"",	"\0"	},
	{	"Kwo",	"",	"\0"	},

	{	"Qya",	"",	"\0"	},
	{	"Qyi",	"",	"\0"	},
	{	"Qyu",	"",	"\0"	},
	{	"Qye",	"",	"\0"	},
	{	"Qyo",	"",	"\0"	},

	{	"Sa",	"",	"\0"	},
	{	"Si",	"",	"\0"	},
	{	"Su",	"",	"\0"	},
	{	"Se",	"",	"\0"	},
	{	"So",	"",	"\0"	},

	{	"Ci",	"",	"\0"	},
	{	"Ce",	"",	"\0"	},

	{	"Sya",	"",	"\0"	},
	{	"Syi",	"",	"\0"	},
	{	"Syu",	"",	"\0"	},
	{	"Sye",	"",	"\0"	},
	{	"Syo",	"",	"\0"	},

	{	"Sha",	"",	"\0"	},
	{	"Shi",	""  ,	"\0"	},
	{	"Shu",	"",	"\0"	},
	{	"She",	"",	"\0"	},
	{	"Sho",	"",	"\0"	},

	{	"Ta",	"",	"\0"	},
	{	"Ti",	"",	"\0"	},
	{	"Tu",	"",	"\0"	},
	{	"Te",	"",	"\0"	},
	{	"To",	"",	"\0"	},

	{	"Ltu",	"",	"\0"	},
	{	"Ltsu",	"",	"\0"	},

	{	"Tya",	"",	"\0"	},
	{	"Tyi",	"",	"\0"	},
	{	"Tyu",	"",	"\0"	},
	{	"Tye",	"",	"\0"	},
	{	"Tyo",	"",	"\0"	},

	{	"Cya",	"",	"\0"	},
	{	"Cyi",	"",	"\0"	},
	{	"Cyu",	"",	"\0"	},
	{	"Cye",	"",	"\0"	},
	{	"Cyo",	"",	"\0"	},

	{	"Cha",	"",	"\0"	},
	{	"Chi",	""  ,	"\0"	},
	{	"Chu",	"",	"\0"	},
	{	"Che",	"",	"\0"	},
	{	"Cho",	"",	"\0"	},

	{	"Tsa",	"Ĥ",	"\0"	},
	{	"Tsi",	"Ĥ",	"\0"	},
	{	"Tsu",	""  ,	"\0"	},
	{	"Tse",	"Ĥ",	"\0"	},
	{	"Tso",	"Ĥ",	"\0"	},

	{	"Tha",	"Ƥ",	"\0"	},
	{	"Thi",	"Ƥ",	"\0"	},
	{	"Thu",	"Ƥ",	"\0"	},
	{	"The",	"Ƥ",	"\0"	},
	{	"Tho",	"Ƥ",	"\0"	},

	{	"Twu",	"Ȥ",	"\0"	},

	{	"Na",	"",	"\0"	},
	{	"Ni",	"",	"\0"	},
	{	"Nu",	"",	"\0"	},
	{	"Ne",	"",	"\0"	},
	{	"No",	"",	"\0"	},

	{	"Nya",	"ˤ",	"\0"	},
	{	"Nyi",	"ˤ",	"\0"	},
	{	"Nyu",	"ˤ",	"\0"	},
	{	"Nye",	"ˤ",	"\0"	},
	{	"Nyo",	"ˤ",	"\0"	},

	{	"Ha",	"",	"\0"	},
	{	"Hi",	"",	"\0"	},
	{	"Hu",	"",	"\0"	},
	{	"He",	"",	"\0"	},
	{	"Ho",	"",	"\0"	},

	{	"Hya",	"Ҥ",	"\0"	},
	{	"Hyi",	"Ҥ",	"\0"	},
	{	"Hyu",	"Ҥ",	"\0"	},
	{	"Hye",	"Ҥ",	"\0"	},
	{	"Hyo",	"Ҥ",	"\0"	},

	{	"Fa",	"դ",	"\0"	},
	{	"Fi",	"դ",	"\0"	},
	{	"Fu",	""  ,	"\0"	},
	{	"Fe",	"դ",	"\0"	},
	{	"Fo",	"դ",	"\0"	},

	{	"Fya",	"դ",	"\0"	},
	{	"Fyi",	"դ",	"\0"	},
	{	"Fyu",	"դ",	"\0"	},
	{	"Fye",	"դ",	"\0"	},
	{	"Fyo",	"դ",	"\0"	},

	{	"Ma",	"",	"\0"	},
	{	"Mi",	"",	"\0"	},
	{	"Mu",	"",	"\0"	},
	{	"Me",	"",	"\0"	},
	{	"Mo",	"",	"\0"	},

	{	"Mya",	"ߤ",	"\0"	},
	{	"Myi",	"ߤ",	"\0"	},
	{	"Myu",	"ߤ",	"\0"	},
	{	"Mye",	"ߤ",	"\0"	},
	{	"Myo",	"ߤ",	"\0"	},

	{	"Ya",	"",	"\0"	},
	{	"Yi",	"",	"\0"	},
	{	"Yu",	"",	"\0"	},
	{	"Ye",	"",	"\0"	},
	{	"Yo",	"",	"\0"	},

	{	"Lya",	"",	"\0"	},
	{	"Lye",	"ꤣ",	"\0"	},
	{	"Lyi",	"",	"\0"	},
	{	"Lyo",	"ꤧ",	"\0"	},
	{	"Lyu",	"",	"\0"	},

	{	"Xya",	"",	"\0"	},
	{	"Xyi",	"",	"\0"	},
	{	"Xyu",	"",	"\0"	},
	{	"Xye",	"",	"\0"	},
	{	"Xyo",	"",	"\0"	},

	{	"Ra",	"",	"\0"	},
	{	"Ri",	"",	"\0"	},
	{	"Ru",	"",	"\0"	},
	{	"Re",	"",	"\0"	},
	{	"Ro",	"",	"\0"	},

	{	"La",	"",	"\0"	},
	{	"Li",	"",	"\0"	},
	{	"Lu",	"",	"\0"	},
	{	"Le",	"",	"\0"	},
	{	"Lo",	"",	"\0"	},

	{	"Rya",	"",	"\0"	},
	{	"Ryi",	"ꤣ",	"\0"	},
	{	"Ryu",	"",	"\0"	},
	{	"Rye",	"ꤧ",	"\0"	},
	{	"Ryo",	"",	"\0"	},

	{	"Wa",	"",	"\0"	},
	{	"Wi",	"",	"\0"	},
	{	"Wu",	"",	"\0"	},
	{	"We",	"",	"\0"	},
	{	"Wo",	"",	"\0"	},

	{	"Wha",	"",	"\0"	},
	{	"Whi",	"",	"\0"	},
	{	"Whu",	""  ,	"\0"	},
	{	"Whe",	"",	"\0"	},
	{	"Who",	"",	"\0"	},

	{	"Xwa",	"",	"\0"	},
	{	"Xtu",	"",	"\0"	},
	{	"Xtsu",	"",	"\0"	},

	{	"Wa",	"",	"\0"	},
	{	"Tu",	"",	"\0"	},
	{	"Tsu",	"",	"\0"	},
	{	"Ya",	"",	"\0"	},
	{	"Yu",	"",	"\0"	},
	{	"Yo",	"",	"\0"	},
	{	"Ll",	"",	"l"		},

	{	"Nn",	"",	"\0"	},
	{	"Mn",	"",	"\0"	},

	{	"Ga",	"",	"\0"	},
	{	"Gi",	"",	"\0"	},
	{	"Gu",	"",	"\0"	},
	{	"Ge",	"",	"\0"	},
	{	"Go",	"",	"\0"	},

	{	"Gya",	"",	"\0"	},
	{	"Gyi",	"",	"\0"	},
	{	"Gyu",	"",	"\0"	},
	{	"Gye",	"",	"\0"	},
	{	"Gyo",	"",	"\0"	},

	{	"Gwa",	"",	"\0"	},
	{	"Gwi",	"",	"\0"	},
	{	"Gwu",	"",	"\0"	},
	{	"Gwe",	"",	"\0"	},
	{	"Gwo",	"",	"\0"	},

	{	"Za",	"",	"\0"	},
	{	"Zi",	"",	"\0"	},
	{	"Zu",	"",	"\0"	},
	{	"Ze",	"",	"\0"	},
	{	"Zo",	"",	"\0"	},

	{	"Jya",	"",	"\0"	},
	{	"Jyi",	"",	"\0"	},
	{	"Jyu",	"",	"\0"	},
	{	"Jye",	"",	"\0"	},
	{	"Jyo",	"",	"\0"	},

	{	"Zya",	"",	"\0"	},
	{	"Zyi",	"",	"\0"	},
	{	"Zyu",	"",	"\0"	},
	{	"Zye",	"",	"\0"	},
	{	"Zyo",	"",	"\0"	},

	{	"Ja",	"",	"\0"	},
	{	"Ji",	""  ,	"\0"	},
	{	"Ju",	"",	"\0"	},
	{	"Je",	"",	"\0"	},
	{	"Jo",	"",	"\0"	},

	{	"Da",	"",	"\0"	},
	{	"Di",	"",	"\0"	},
	{	"Du",	"",	"\0"	},
	{	"De",	"",	"\0"	},
	{	"Do",	"",	"\0"	},

	{	"Dya",	"¤",	"\0"	},
	{	"Dyi",	"¤",	"\0"	},
	{	"Dyu",	"¤",	"\0"	},
	{	"Dye",	"¤",	"\0"	},
	{	"Dyo",	"¤",	"\0"	},

	{	"Dha",	"Ǥ",	"\0"	},
	{	"Dhi",	"Ǥ",	"\0"	},
	{	"Dhu",	"Ǥ",	"\0"	},
	{	"Dhe",	"Ǥ",	"\0"	},
	{	"Dho",	"Ǥ",	"\0"	},

	{	"Dwu",	"ɤ",	"\0"	},

	{	"Ba",	"",	"\0"	},
	{	"Bi",	"",	"\0"	},
	{	"Bu",	"",	"\0"	},
	{	"Be",	"",	"\0"	},
	{	"Bo",	"",	"\0"	},

	{	"Bya",	"Ӥ",	"\0"	},
	{	"Byi",	"Ӥ",	"\0"	},
	{	"Byu",	"Ӥ",	"\0"	},
	{	"Bye",	"Ӥ",	"\0"	},
	{	"Byo",	"Ӥ",	"\0"	},

	{	"Pa",	"",	"\0"	},
	{	"Pi",	"",	"\0"	},
	{	"Pu",	"",	"\0"	},
	{	"Pe",	"",	"\0"	},
	{	"Po",	"",	"\0"	},

	{	"Pya",	"Ԥ",	"\0"	},
	{	"Pyi",	"Ԥ",	"\0"	},
	{	"Pyu",	"Ԥ",	"\0"	},
	{	"Pye",	"Ԥ",	"\0"	},
	{	"Pyo",	"Ԥ",	"\0"	},

	{	"Ca",	"",	"\0"	},
	{	"Cu",	"",	"\0"	},
	{	"Co",	"",	"\0"	},

	{	"Xka",	"",	"\0"	},
	{	"Xke",	"",	"\0"	},

	{	"Va",	"",	"\0"	},
	{	"Vi",	"",	"\0"	},
	{	"Vu",	""  ,	"\0"	},
	{	"Ve",	"",	"\0"	},
	{	"Vo",	"",	"\0"	},

	{	"Vya",	"",	"\0"	},
	{	"Vyi",	"",	"\0"	},
	{	"Vyu",	"",	"\0"	},
	{	"Vye",	"",	"\0"	},
	{	"Vyo",	"",	"\0"	},

	{	"Kk",	"",	"k"	},
	{	"Ss",	"",	"s"	},
	{	"Tt",	"",	"t"	},
	{	"Hh",	"",	"h"	},
	{	"Mm",	"",	"m"	},
	{	"Yy",	"",	"y"	},
	{	"Rr",	"",	"r"	},
	{	"Ww",	"",	"w"	},
	{	"Gg",	"",	"g"	},
	{	"Zz",	"",	"z"	},
	{	"Dd",	"",	"d"	},
	{	"Bb",	"",	"b"	},
	{	"Pp",	"",	"p"	},
	{	"Cc",	"",	"c"	},
	{	"Ff",	"",	"f"	},
	{	"Jj",	"",	"j"	},
	{	"Qq",	"",	"q"	},
	{	"Vv",	"",	"v"	},
	{	"Tch",	"",	"ch"	},

	{	"Z ",	"",	"\0"	},
	{	"@ ",	"",	"\0"	},
	{	"ZA",	"",	"\0"	},
	{	"ZB",	"",	"\0"	},
	{	"ZC",	"",	"\0"	},
	{	"ZD",	"",	"\0"	},
	{	"ZE",	"",	"\0"	},
	{	"ZF",	"",	"\0"	},
	{	"ZG",	"",	"\0"	},
	{	"ZH",	"",	"\0"	},
	{	"ZI",	"",	"\0"	},
	{	"ZJ",	"",	"\0"	},
	{	"ZK",	"",	"\0"	},
	{	"ZL",	"",	"\0"	},
	{	"ZM",	"",	"\0"	},
	{	"ZN",	"",	"\0"	},
	{	"ZO",	"",	"\0"	},
	{	"ZP",	"",	"\0"	},
	{	"ZQ",	"",	"\0"	},
	{	"ZR",	"",	"\0"	},
	{	"ZS",	"",	"\0"	},
	{	"ZT",	"",	"\0"	},
	{	"ZU",	"",	"\0"	},
	{	"ZV",	"",	"\0"	},
	{	"ZW",	"",	"\0"	},
	{	"ZX",	"",	"\0"	},
	{	"ZY",	"",	"\0"	},
	{	"ZZ",	"",	"\0"	},
	{	"Za",	"",	"\0"	},
	{	"Zb",	"",	"\0"	},
	{	"Zc",	"",	"\0"	},
	{	"Zd",	"",	"\0"	},
	{	"Ze",	"",	"\0"	},
	{	"Zf",	"",	"\0"	},
	{	"Zg",	"",	"\0"	},
	{	"Zh",	"",	"\0"	},
	{	"Zi",	"",	"\0"	},
	{	"Zj",	"",	"\0"	},
	{	"Zk",	"",	"\0"	},
	{	"Zl",	"",	"\0"	},
	{	"Zm",	"",	"\0"	},
	{	"Zn",	"",	"\0"	},
	{	"Zo",	"",	"\0"	},
	{	"Zp",	"",	"\0"	},
	{	"Zq",	"",	"\0"	},
	{	"Zr",	"",	"\0"	},
	{	"Zs",	"",	"\0"	},
	{	"Zt",	"",	"\0"	},
	{	"Zu",	"",	"\0"	},
	{	"Zv",	"",	"\0"	},
	{	"Zw",	"",	"\0"	},
	{	"Zx",	"",	"\0"	},
	{	"Zy",	"",	"\0"	},
	{	"Zz",	"",	"\0"	},
	{	"Z0",	"",	"\0"	},
	{	"Z1",	"",	"\0"	},
	{	"Z2",	"",	"\0"	},
	{	"Z3",	"",	"\0"	},
	{	"Z4",	"",	"\0"	},
	{	"Z5",	"",	"\0"	},
	{	"Z6",	"",	"\0"	},
	{	"Z7",	"",	"\0"	},
	{	"Z8",	"",	"\0"	},
	{	"Z9",	"",	"\0"	},
	{	"Z!",	"",	"\0"	},
	{	"Z\"",	"",	"\0"	},
	{	"Z#",	"",	"\0"	},
	{	"Z$",	"",	"\0"	},
	{	"Z%",	"",	"\0"	},
	{	"Z&",	"",	"\0"	},
	{	"Z'",	"",	"\0"	},
	{	"Z(",	"",	"\0"	},
	{	"Z)",	"",	"\0"	},
	{	"Z~",	"",	"\0"	},
	{	"Z-",	"",	"\0"	},
	{	"Z=",	"",	"\0"	},
	{	"Z^",	"",	"\0"	},
	{	"Z|",	"",	"\0"	},
	{	"Z@",	"",	"\0"	},
	{	"Z`",	"",	"\0"	},
	{	"Z[",	"",	"\0"	},
	{	"Z{",	"",	"\0"	},
	{	"Z;",	"",	"\0"	},
	{	"Z+",	"",	"\0"	},
	{	"Z:",	"",	"\0"	},
	{	"Z*",	"",	"\0"	},
	{	"Z]",	"",	"\0"	},
	{	"Z}",	"",	"\0"	},
	{	"Z<",	"",	"\0"	},
	{	"Z,",	"",	"\0"	},
	{	"Z>",	"",	"\0"	},
	{	"Z.",	"",	"\0"	},
	{	"Z/",	"",	"\0"	},
	{	"Z?",	"",	"\0"	},
	{	"Z\\",	"",	"\0"	},
	{	"Z_",	"",	"\0"	},

	{	",",	"",	"\0"	},
	{	".",	"",	"\0"	},
	{	"[",	"",	"\0"	},
	{	"]",	"",	"\0"	},
	{	"{",	"",	"\0"	},
	{	"}",	"",	"\0"	},
	{	"^",	"",	"\0"	},
	{	"<",	"",	"\0"	},
	{	">",	"",	"\0"	},
	{	"-",	"",	"\0"	},
	{	"@-",	"",	"\0"	},
	{	"@@",	"",	"\0"	},
	{	"@/",	"",	"\0"	},
	{	"@\\",	"",	"\0"	},
	{	"@",	"",	"\0"	},
	{	"@,",	"",	"\0"	},
	{	"@.",	"",	"\0"	},
	{	"@~",	"",	"\0"	},
	{	"@||",	"",	"\0"	},
	{	"@|",	"",	"\0"	},
	{	"@3",	"",	"\0"	},
	{	"@2",	"",	"\0"	},
	{	"@(",	"",	"\0"	},
	{	"@)",	"",	"\0"	},
	{	"@[",	"",	"\0"	},
	{	"@]",	"",	"\0"	},
	{	"@{",	"",	"\0"	},
	{	"@}",	"",	"\0"	},

	{	"~",	"",	"\0"	},
	{	"?",	"",	"\0"	},
	{	"!",	"",	"\0"	},
	{	"#",	"",	"\0"	},
	{	"$",	"",	"\0"	},
	{	"%",	"",	"\0"	},
	{	"&",	"",	"\0"	},
	{	"`",	"",	"\0"	},
	{	"+",	"",	"\0"	},
	{	"*",	"",	"\0"	},
	{	"=",	"",	"\0"	},
	{	"|",	"",	"\0"	},
	{	";",	"",	"\0"	},
	{	":",	"",	"\0"	},
	{	"_",	"",	"\0"	},
	{	"/",	"",	"\0"	},
	{	"\\",	"",	"\0"	},

	{	NULL,	NULL,	NULL	},
};


static const struct rk_map_source_t rk_map_katakana[] = {
	{	"1",	"",	"\0"	},
	{	"2",	"",	"\0"	},
	{	"3",	"",	"\0"	},
	{	"4",	"",	"\0"	},
	{	"5",	"",	"\0"	},
	{	"6",	"",	"\0"	},
	{	"7",	"",	"\0"	},
	{	"8",	"",	"\0"	},
	{	"9",	"",	"\0"	},
	{	"0",	"",	"\0"	},

	{	"a",	"",	"\0"	},
	{	"i",	"",	"\0"	},
	{	"u",	"",	"\0"	},
	{	"e",	"",	"\0"	},
	{	"o",	"",	"\0"	},

	{	"xa",	"",	"\0"	},
	{	"xi",	"",	"\0"	},
	{	"xu",	"",	"\0"	},
	{	"xe",	"",	"\0"	},
	{	"xo",	"",	"\0"	},

	{	"ka",	"",	"\0"	},
	{	"ki",	"",	"\0"	},
	{	"ku",	"",	"\0"	},
	{	"ke",	"",	"\0"	},
	{	"ko",	"",	"\0"	},

	{	"kya",	"",	"\0"	},
	{	"kyi",	"",	"\0"	},
	{	"kyu",	"",	"\0"	},
	{	"kye",	"",	"\0"	},
	{	"kyo",	"",	"\0"	},

	{	"kwa",	"",	"\0"	},
	{	"kwi",	"",	"\0"	},
	{	"kwu",	""  ,	"\0"	},
	{	"kwe",	"",	"\0"	},
	{	"kwo",	"",	"\0"	},

	{	"qya",	"",	"\0"	},
	{	"qyi",	"",	"\0"	},
	{	"qyu",	"",	"\0"	},
	{	"qye",	"",	"\0"	},
	{	"qyo",	"",	"\0"	},

	{	"sa",	"",	"\0"	},
	{	"si",	"",	"\0"	},
	{	"su",	"",	"\0"	},
	{	"se",	"",	"\0"	},
	{	"so",	"",	"\0"	},

	{	"ci",	"",	"\0"	},
	{	"ce",	"",	"\0"	},

	{	"sya",	"",	"\0"	},
	{	"syi",	"",	"\0"	},
	{	"syu",	"",	"\0"	},
	{	"sye",	"",	"\0"	},
	{	"syo",	"",	"\0"	},

	{	"sha",	"",	"\0"	},
	{	"shi",	""  ,	"\0"	},
	{	"shu",	"",	"\0"	},
	{	"she",	"",	"\0"	},
	{	"sho",	"",	"\0"	},

	{	"ta",	"",	"\0"	},
	{	"ti",	"",	"\0"	},
	{	"tu",	"",	"\0"	},
	{	"te",	"",	"\0"	},
	{	"to",	"",	"\0"	},

	{	"ltu",	"",	"\0"	},
	{	"ltsu",	"",	"\0"	},

	{	"tya",	"",	"\0"	},
	{	"tyi",	"",	"\0"	},
	{	"tyu",	"",	"\0"	},
	{	"tye",	"",	"\0"	},
	{	"tyo",	"",	"\0"	},

	{	"cya",	"",	"\0"	},
	{	"cyi",	"",	"\0"	},
	{	"cyu",	"",	"\0"	},
	{	"cye",	"",	"\0"	},
	{	"cyo",	"",	"\0"	},

	{	"cha",	"",	"\0"	},
	{	"chi",	""  ,	"\0"	},
	{	"chu",	"",	"\0"	},
	{	"che",	"",	"\0"	},
	{	"cho",	"",	"\0"	},

	{	"tsa",	"ĥ",	"\0"	},
	{	"tsi",	"ĥ",	"\0"	},
	{	"tsu",	""  ,	"\0"	},
	{	"tse",	"ĥ",	"\0"	},
	{	"tso",	"ĥ",	"\0"	},

	{	"tha",	"ƥ",	"\0"	},
	{	"thi",	"ƥ",	"\0"	},
	{	"thu",	"ƥ",	"\0"	},
	{	"the",	"ƥ",	"\0"	},
	{	"tho",	"ƥ",	"\0"	},

	{	"twu",	"ȥ",	"\0"	},

	{	"na",	"",	"\0"	},
	{	"ni",	"",	"\0"	},
	{	"nu",	"",	"\0"	},
	{	"ne",	"",	"\0"	},
	{	"no",	"",	"\0"	},

	{	"nya",	"˥",	"\0"	},
	{	"nyi",	"˥",	"\0"	},
	{	"nyu",	"˥",	"\0"	},
	{	"nye",	"˥",	"\0"	},
	{	"nyo",	"˥",	"\0"	},

	{	"ha",	"",	"\0"	},
	{	"hi",	"",	"\0"	},
	{	"hu",	"",	"\0"	},
	{	"he",	"",	"\0"	},
	{	"ho",	"",	"\0"	},

	{	"hya",	"ҥ",	"\0"	},
	{	"hyi",	"ҥ",	"\0"	},
	{	"hyu",	"ҥ",	"\0"	},
	{	"hye",	"ҥ",	"\0"	},
	{	"hyo",	"ҥ",	"\0"	},

	{	"fa",	"ե",	"\0"	},
	{	"fi",	"ե",	"\0"	},
	{	"fu",	""  ,	"\0"	},
	{	"fe",	"ե",	"\0"	},
	{	"fo",	"ե",	"\0"	},

	{	"fya",	"ե",	"\0"	},
	{	"fyi",	"ե",	"\0"	},
	{	"fyu",	"ե",	"\0"	},
	{	"fye",	"ե",	"\0"	},
	{	"fyo",	"ե",	"\0"	},

	{	"ma",	"",	"\0"	},
	{	"mi",	"",	"\0"	},
	{	"mu",	"",	"\0"	},
	{	"me",	"",	"\0"	},
	{	"mo",	"",	"\0"	},

	{	"mya",	"ߥ",	"\0"	},
	{	"myi",	"ߥ",	"\0"	},
	{	"myu",	"ߥ",	"\0"	},
	{	"mye",	"ߥ",	"\0"	},
	{	"myo",	"ߥ",	"\0"	},

	{	"ya",	"",	"\0"	},
	{	"yi",	"",	"\0"	},
	{	"yu",	"",	"\0"	},
	{	"ye",	"",	"\0"	},
	{	"yo",	"",	"\0"	},

	{	"lya",	"",	"\0"	},
	{	"lye",	"ꥣ",	"\0"	},
	{	"lyi",	"",	"\0"	},
	{	"lyo",	"ꥧ",	"\0"	},
	{	"lyu",	"",	"\0"	},

	{	"xya",	"",	"\0"	},
	{	"xyi",	"",	"\0"	},
	{	"xyu",	"",	"\0"	},
	{	"xye",	"",	"\0"	},
	{	"xyo",	"",	"\0"	},

	{	"ra",	"",	"\0"	},
	{	"ri",	"",	"\0"	},
	{	"ru",	"",	"\0"	},
	{	"re",	"",	"\0"	},
	{	"ro",	"",	"\0"	},

	{	"la",	"",	"\0"	},
	{	"li",	"",	"\0"	},
	{	"lu",	"",	"\0"	},
	{	"le",	"",	"\0"	},
	{	"lo",	"",	"\0"	},

	{	"rya",	"",	"\0"	},
	{	"ryi",	"ꥣ",	"\0"	},
	{	"ryu",	"",	"\0"	},
	{	"rye",	"ꥧ",	"\0"	},
	{	"ryo",	"",	"\0"	},

	{	"wa",	"",	"\0"	},
	{	"wi",	"",	"\0"	},
	{	"wu",	"",	"\0"	},
	{	"we",	"",	"\0"	},
	{	"wo",	"",	"\0"	},

	{	"wha",	"",	"\0"	},
	{	"whi",	"",	"\0"	},
	{	"whu",	""  ,	"\0"	},
	{	"whe",	"",	"\0"	},
	{	"who",	"",	"\0"	},

	{	"xwa",	"",	"\0"	},
	{	"xtu",	"",	"\0"	},
	{	"xtsu",	"",	"\0"	},

	{	"A",	"",	"\0"	},
	{	"I",	"",	"\0"	},
	{	"U",	"",	"\0"	},
	{	"E",	"",	"\0"	},
	{	"O",	"",	"\0"	},

	{	"WA",	"",	"\0"	},
	{	"TU",	"",	"\0"	},
	{	"TSU",	"",	"\0"	},
	{	"YA",	"",	"\0"	},
	{	"YU",	"",	"\0"	},
	{	"YO",	"",	"\0"	},
	{	"ll",	"",	"l"		},

	{	"nn",	"",	"\0"	},
	{	"mn",	"",	"\0"	},

	{	"ga",	"",	"\0"	},
	{	"gi",	"",	"\0"	},
	{	"gu",	"",	"\0"	},
	{	"ge",	"",	"\0"	},
	{	"go",	"",	"\0"	},

	{	"gya",	"",	"\0"	},
	{	"gyi",	"",	"\0"	},
	{	"gyu",	"",	"\0"	},
	{	"gye",	"",	"\0"	},
	{	"gyo",	"",	"\0"	},

	{	"gwa",	"",	"\0"	},
	{	"gwi",	"",	"\0"	},
	{	"gwu",	"",	"\0"	},
	{	"gwe",	"",	"\0"	},
	{	"gwo",	"",	"\0"	},

	{	"za",	"",	"\0"	},
	{	"zi",	"",	"\0"	},
	{	"zu",	"",	"\0"	},
	{	"ze",	"",	"\0"	},
	{	"zo",	"",	"\0"	},

	{	"jya",	"",	"\0"	},
	{	"jyi",	"",	"\0"	},
	{	"jyu",	"",	"\0"	},
	{	"jye",	"",	"\0"	},
	{	"jyo",	"",	"\0"	},

	{	"zya",	"",	"\0"	},
	{	"zyi",	"",	"\0"	},
	{	"zyu",	"",	"\0"	},
	{	"zye",	"",	"\0"	},
	{	"zyo",	"",	"\0"	},

	{	"ja",	"",	"\0"	},
	{	"ji",	""  ,	"\0"	},
	{	"ju",	"",	"\0"	},
	{	"je",	"",	"\0"	},
	{	"jo",	"",	"\0"	},

	{	"da",	"",	"\0"	},
	{	"di",	"",	"\0"	},
	{	"du",	"",	"\0"	},
	{	"de",	"",	"\0"	},
	{	"do",	"",	"\0"	},

	{	"dya",	"¥",	"\0"	},
	{	"dyi",	"¥",	"\0"	},
	{	"dyu",	"¥",	"\0"	},
	{	"dye",	"¥",	"\0"	},
	{	"dyo",	"¥",	"\0"	},

	{	"dha",	"ǥ",	"\0"	},
	{	"dhi",	"ǥ",	"\0"	},
	{	"dhu",	"ǥ",	"\0"	},
	{	"dhe",	"ǥ",	"\0"	},
	{	"dho",	"ǥ",	"\0"	},

	{	"dwu",	"ɥ",	"\0"	},

	{	"ba",	"",	"\0"	},
	{	"bi",	"",	"\0"	},
	{	"bu",	"",	"\0"	},
	{	"be",	"",	"\0"	},
	{	"bo",	"",	"\0"	},

	{	"bya",	"ӥ",	"\0"	},
	{	"byi",	"ӥ",	"\0"	},
	{	"byu",	"ӥ",	"\0"	},
	{	"bye",	"ӥ",	"\0"	},
	{	"byo",	"ӥ",	"\0"	},

	{	"pa",	"",	"\0"	},
	{	"pi",	"",	"\0"	},
	{	"pu",	"",	"\0"	},
	{	"pe",	"",	"\0"	},
	{	"po",	"",	"\0"	},

	{	"pya",	"ԥ",	"\0"	},
	{	"pyi",	"ԥ",	"\0"	},
	{	"pyu",	"ԥ",	"\0"	},
	{	"pye",	"ԥ",	"\0"	},
	{	"pyo",	"ԥ",	"\0"	},

	{	"ca",	"",	"\0"	},
	{	"cu",	"",	"\0"	},
	{	"co",	"",	"\0"	},

	{	"xka",	"",	"\0"	},
	{	"xke",	"",	"\0"	},

	{	"va",	"",	"\0"	},
	{	"vi",	"",	"\0"	},
	{	"vu",	""  ,	"\0"	},
	{	"ve",	"",	"\0"	},
	{	"vo",	"",	"\0"	},

	{	"vya",	"",	"\0"	},
	{	"vyi",	"",	"\0"	},
	{	"vyu",	"",	"\0"	},
	{	"vye",	"",	"\0"	},
	{	"vyo",	"",	"\0"	},

	{	"kk",	"",	"k"	},
	{	"ss",	"",	"s"	},
	{	"tt",	"",	"t"	},
	{	"hh",	"",	"h"	},
	{	"mm",	"",	"m"	},
	{	"yy",	"",	"y"	},
	{	"rr",	"",	"r"	},
	{	"ww",	"",	"w"	},
	{	"gg",	"",	"g"	},
	{	"zz",	"",	"z"	},
	{	"dd",	"",	"d"	},
	{	"bb",	"",	"b"	},
	{	"pp",	"",	"p"	},
	{	"cc",	"",	"c"	},
	{	"ff",	"",	"f"	},
	{	"jj",	"",	"j"	},
	{	"qq",	"",	"q"	},
	{	"vv",	"",	"v"	},
	{	"tch",	"",	"ch"	},

	{	"Z ",	"",	"\0"	},
	{	"@ ",	"",	"\0"	},
	{	"ZA",	"",	"\0"	},
	{	"ZB",	"",	"\0"	},
	{	"ZC",	"",	"\0"	},
	{	"ZD",	"",	"\0"	},
	{	"ZE",	"",	"\0"	},
	{	"ZF",	"",	"\0"	},
	{	"ZG",	"",	"\0"	},
	{	"ZH",	"",	"\0"	},
	{	"ZI",	"",	"\0"	},
	{	"ZJ",	"",	"\0"	},
	{	"ZK",	"",	"\0"	},
	{	"ZL",	"",	"\0"	},
	{	"ZM",	"",	"\0"	},
	{	"ZN",	"",	"\0"	},
	{	"ZO",	"",	"\0"	},
	{	"ZP",	"",	"\0"	},
	{	"ZQ",	"",	"\0"	},
	{	"ZR",	"",	"\0"	},
	{	"ZS",	"",	"\0"	},
	{	"ZT",	"",	"\0"	},
	{	"ZU",	"",	"\0"	},
	{	"ZV",	"",	"\0"	},
	{	"ZW",	"",	"\0"	},
	{	"ZX",	"",	"\0"	},
	{	"ZY",	"",	"\0"	},
	{	"ZZ",	"",	"\0"	},
	{	"Za",	"",	"\0"	},
	{	"Zb",	"",	"\0"	},
	{	"Zc",	"",	"\0"	},
	{	"Zd",	"",	"\0"	},
	{	"Ze",	"",	"\0"	},
	{	"Zf",	"",	"\0"	},
	{	"Zg",	"",	"\0"	},
	{	"Zh",	"",	"\0"	},
	{	"Zi",	"",	"\0"	},
	{	"Zj",	"",	"\0"	},
	{	"Zk",	"",	"\0"	},
	{	"Zl",	"",	"\0"	},
	{	"Zm",	"",	"\0"	},
	{	"Zn",	"",	"\0"	},
	{	"Zo",	"",	"\0"	},
	{	"Zp",	"",	"\0"	},
	{	"Zq",	"",	"\0"	},
	{	"Zr",	"",	"\0"	},
	{	"Zs",	"",	"\0"	},
	{	"Zt",	"",	"\0"	},
	{	"Zu",	"",	"\0"	},
	{	"Zv",	"",	"\0"	},
	{	"Zw",	"",	"\0"	},
	{	"Zx",	"",	"\0"	},
	{	"Zy",	"",	"\0"	},
	{	"Zz",	"",	"\0"	},
	{	"Z0",	"",	"\0"	},
	{	"Z1",	"",	"\0"	},
	{	"Z2",	"",	"\0"	},
	{	"Z3",	"",	"\0"	},
	{	"Z4",	"",	"\0"	},
	{	"Z5",	"",	"\0"	},
	{	"Z6",	"",	"\0"	},
	{	"Z7",	"",	"\0"	},
	{	"Z8",	"",	"\0"	},
	{	"Z9",	"",	"\0"	},
	{	"Z!",	"",	"\0"	},
	{	"Z\"",	"",	"\0"	},
	{	"Z#",	"",	"\0"	},
	{	"Z$",	"",	"\0"	},
	{	"Z%",	"",	"\0"	},
	{	"Z&",	"",	"\0"	},
	{	"Z'",	"",	"\0"	},
	{	"Z(",	"",	"\0"	},
	{	"Z)",	"",	"\0"	},
	{	"Z~",	"",	"\0"	},
	{	"Z-",	"",	"\0"	},
	{	"Z=",	"",	"\0"	},
	{	"Z^",	"",	"\0"	},
	{	"Z|",	"",	"\0"	},
	{	"Z@",	"",	"\0"	},
	{	"Z`",	"",	"\0"	},
	{	"Z[",	"",	"\0"	},
	{	"Z{",	"",	"\0"	},
	{	"Z;",	"",	"\0"	},
	{	"Z+",	"",	"\0"	},
	{	"Z:",	"",	"\0"	},
	{	"Z*",	"",	"\0"	},
	{	"Z]",	"",	"\0"	},
	{	"Z}",	"",	"\0"	},
	{	"Z<",	"",	"\0"	},
	{	"Z,",	"",	"\0"	},
	{	"Z>",	"",	"\0"	},
	{	"Z.",	"",	"\0"	},
	{	"Z/",	"",	"\0"	},
	{	"Z?",	"",	"\0"	},
	{	"Z\\",	"",	"\0"	},
	{	"Z_",	"",	"\0"	},

	{	",",	"",	"\0"	},
	{	".",	"",	"\0"	},
	{	"[",	"",	"\0"	},
	{	"]",	"",	"\0"	},
	{	"{",	"",	"\0"	},
	{	"}",	"",	"\0"	},
	{	"^",	"",	"\0"	},
	{	"<",	"",	"\0"	},
	{	">",	"",	"\0"	},
	{	"-",	"",	"\0"	},
	{	"@-",	"",	"\0"	},
	{	"@@",	"",	"\0"	},
	{	"@/",	"",	"\0"	},
	{	"@\\",	"",	"\0"	},
	{	"@",	"",	"\0"	},
	{	"@,",	"",	"\0"	},
	{	"@.",	"",	"\0"	},
	{	"@~",	"",	"\0"	},
	{	"@||",	"",	"\0"	},
	{	"@|",	"",	"\0"	},
	{	"@3",	"",	"\0"	},
	{	"@2",	"",	"\0"	},
	{	"@(",	"",	"\0"	},
	{	"@)",	"",	"\0"	},
	{	"@[",	"",	"\0"	},
	{	"@]",	"",	"\0"	},
	{	"@{",	"",	"\0"	},
	{	"@}",	"",	"\0"	},

	{	"~",	"",	"\0"	},
	{	"?",	"",	"\0"	},
	{	"!",	"",	"\0"	},
	{	"#",	"",	"\0"	},
	{	"$",	"",	"\0"	},
	{	"%",	"",	"\0"	},
	{	"&",	"",	"\0"	},
	{	"`",	"",	"\0"	},
	{	"+",	"",	"\0"	},
	{	"*",	"",	"\0"	},
	{	"=",	"",	"\0"	},
	{	"|",	"",	"\0"	},
	{	";",	"",	"\0"	},
	{	":",	"",	"\0"	},
	{	"_",	"",	"\0"	},
	{	"/",	"",	"\0"	},
	{	"\\",	"",	"\0"	},

	{	NULL,	NULL,	NULL	},
};


static const struct rk_map_source_t rk_map_alphabet[] = {
	{	" ",	" ",	"\0"	},
	{	"!",	"!",	"\0"	},
	{	"\"",	"\"",	"\0"	},
	{	"#",	"#",	"\0"	},
	{	"$",	"$",	"\0"	},
	{	"%",	"%",	"\0"	},
	{	"&",	"&",	"\0"	},
	{	"'",	"'",	"\0"	},
	{	"(",	"(",	"\0"	},
	{	")",	")",	"\0"	},
	{	"*",	"*",	"\0"	},
	{	"+",	"+",	"\0"	},
	{	",",	",",	"\0"	},
	{	"-",	"-",	"\0"	},
	{	".",	".",	"\0"	},
	{	"/",	"/",	"\0"	},

	{	"0",	"0",	"\0"	},
	{	"1",	"1",	"\0"	},
	{	"2",	"2",	"\0"	},
	{	"3",	"3",	"\0"	},
	{	"4",	"4",	"\0"	},
	{	"5",	"5",	"\0"	},
	{	"6",	"6",	"\0"	},
	{	"7",	"7",	"\0"	},
	{	"8",	"8",	"\0"	},
	{	"9",	"9",	"\0"	},
	{	":",	":",	"\0"	},
	{	";",	";",	"\0"	},
	{	"<",	"<",	"\0"	},
	{	"=",	"=",	"\0"	},
	{	">",	">",	"\0"	},
	{	"?",	"?",	"\0"	},

	{	"@",	"@",	"\0"	},
	{	"A",	"A",	"\0"	},
	{	"B",	"B",	"\0"	},
	{	"C",	"C",	"\0"	},
	{	"D",	"D",	"\0"	},
	{	"E",	"E",	"\0"	},
	{	"F",	"F",	"\0"	},
	{	"G",	"G",	"\0"	},
	{	"H",	"H",	"\0"	},
	{	"I",	"I",	"\0"	},
	{	"J",	"J",	"\0"	},
	{	"K",	"K",	"\0"	},
	{	"L",	"L",	"\0"	},
	{	"M",	"M",	"\0"	},
	{	"N",	"N",	"\0"	},
	{	"O",	"O",	"\0"	},
	{	"P",	"P",	"\0"	},
	{	"Q",	"Q",	"\0"	},
	{	"R",	"R",	"\0"	},
	{	"S",	"S",	"\0"	},
	{	"T",	"T",	"\0"	},
	{	"U",	"U",	"\0"	},
	{	"V",	"V",	"\0"	},
	{	"W",	"W",	"\0"	},
	{	"X",	"X",	"\0"	},
	{	"Y",	"Y",	"\0"	},
	{	"Z",	"Z",	"\0"	},
	{	"[",	"[",	"\0"	},
	{	"\\",	"\\",	"\0"	},
	{	"]",	"]",	"\0"	},
	{	"^",	"^",	"\0"	},
	{	"_",	"_",	"\0"	},

	{	"`",	"`",	"\0"	},
	{	"a",	"a",	"\0"	},
	{	"b",	"b",	"\0"	},
	{	"c",	"c",	"\0"	},
	{	"d",	"d",	"\0"	},
	{	"e",	"e",	"\0"	},
	{	"f",	"f",	"\0"	},
	{	"g",	"g",	"\0"	},
	{	"h",	"h",	"\0"	},
	{	"i",	"i",	"\0"	},
	{	"j",	"j",	"\0"	},
	{	"k",	"k",	"\0"	},
	{	"l",	"l",	"\0"	},
	{	"m",	"m",	"\0"	},
	{	"n",	"n",	"\0"	},
	{	"o",	"o",	"\0"	},
	{	"p",	"p",	"\0"	},
	{	"q",	"q",	"\0"	},
	{	"r",	"r",	"\0"	},
	{	"s",	"s",	"\0"	},
	{	"t",	"t",	"\0"	},
	{	"u",	"u",	"\0"	},
	{	"v",	"v",	"\0"	},
	{	"w",	"w",	"\0"	},
	{	"x",	"x",	"\0"	},
	{	"y",	"y",	"\0"	},
	{	"z",	"z",	"\0"	},
	{	"{",	"{",	"\0"	},
	{	"|",	"|",	"\0"	},
	{	"}",	"}",	"\0"	},
	{	"~",	"~",	"\0"	},

	{	NULL,	NULL,	NULL	},
};


static const struct rk_map_source_t rk_map_walphabet[] = {
	{	" ",	"",	"\0"	},
	{	"!",	"",	"\0"	},
	{	"\"",	"",	"\0"	},
	{	"#",	"",	"\0"	},
	{	"$",	"",	"\0"	},
	{	"%",	"",	"\0"	},
	{	"&",	"",	"\0"	},
	{	"'",	"",	"\0"	},
	{	"(",	"",	"\0"	},
	{	")",	"",	"\0"	},
	{	"*",	"",	"\0"	},
	{	"+",	"",	"\0"	},
	{	",",	"",	"\0"	},
	{	"-",	"",	"\0"	},
	{	".",	"",	"\0"	},
	{	"/",	"",	"\0"	},

	{	"0",	"",	"\0"	},
	{	"1",	"",	"\0"	},
	{	"2",	"",	"\0"	},
	{	"3",	"",	"\0"	},
	{	"4",	"",	"\0"	},
	{	"5",	"",	"\0"	},
	{	"6",	"",	"\0"	},
	{	"7",	"",	"\0"	},
	{	"8",	"",	"\0"	},
	{	"9",	"",	"\0"	},
	{	":",	"",	"\0"	},
	{	";",	"",	"\0"	},
	{	"<",	"",	"\0"	},
	{	"=",	"",	"\0"	},
	{	">",	"",	"\0"	},
	{	"?",	"",	"\0"	},

	{	"@",	"",	"\0"	},
	{	"A",	"",	"\0"	},
	{	"B",	"",	"\0"	},
	{	"C",	"",	"\0"	},
	{	"D",	"",	"\0"	},
	{	"E",	"",	"\0"	},
	{	"F",	"",	"\0"	},
	{	"G",	"",	"\0"	},
	{	"H",	"",	"\0"	},
	{	"I",	"",	"\0"	},
	{	"J",	"",	"\0"	},
	{	"K",	"",	"\0"	},
	{	"L",	"",	"\0"	},
	{	"M",	"",	"\0"	},
	{	"N",	"",	"\0"	},
	{	"O",	"",	"\0"	},
	{	"P",	"",	"\0"	},
	{	"Q",	"",	"\0"	},
	{	"R",	"",	"\0"	},
	{	"S",	"",	"\0"	},
	{	"T",	"",	"\0"	},
	{	"U",	"",	"\0"	},
	{	"V",	"",	"\0"	},
	{	"W",	"",	"\0"	},
	{	"X",	"",	"\0"	},
	{	"Y",	"",	"\0"	},
	{	"Z",	"",	"\0"	},
	{	"[",	"",	"\0"	},
	{	"\\",	"",	"\0"	},
	{	"]",	"",	"\0"	},
	{	"^",	"",	"\0"	},
	{	"_",	"",	"\0"	},

	{	"`",	"",	"\0"	},
	{	"a",	"",	"\0"	},
	{	"b",	"",	"\0"	},
	{	"c",	"",	"\0"	},
	{	"d",	"",	"\0"	},
	{	"e",	"",	"\0"	},
	{	"f",	"",	"\0"	},
	{	"g",	"",	"\0"	},
	{	"h",	"",	"\0"	},
	{	"i",	"",	"\0"	},
	{	"j",	"",	"\0"	},
	{	"k",	"",	"\0"	},
	{	"l",	"",	"\0"	},
	{	"m",	"",	"\0"	},
	{	"n",	"",	"\0"	},
	{	"o",	"",	"\0"	},
	{	"p",	"",	"\0"	},
	{	"q",	"",	"\0"	},
	{	"r",	"",	"\0"	},
	{	"s",	"",	"\0"	},
	{	"t",	"",	"\0"	},
	{	"u",	"",	"\0"	},
	{	"v",	"",	"\0"	},
	{	"w",	"",	"\0"	},
	{	"x",	"",	"\0"	},
	{	"y",	"",	"\0"	},
	{	"z",	"",	"\0"	},
	{	"{",	"",	"\0"	},
	{	"|",	"",	"\0"	},
	{	"}",	"",	"\0"	},
	{	"~",	"",	"\0"	},

	{	NULL,	NULL,	NULL	},
};


static const struct rk_map_source_t rk_map_hankaku_kana[] = {
	{	"1",	"1",	"\0"	},
	{	"2",	"2",	"\0"	},
	{	"3",	"3",	"\0"	},
	{	"4",	"4",	"\0"	},
	{	"5",	"5",	"\0"	},
	{	"6",	"6",	"\0"	},
	{	"7",	"7",	"\0"	},
	{	"8",	"8",	"\0"	},
	{	"9",	"9",	"\0"	},
	{	"0",	"0",	"\0"	},

	{	"a",	"",	"\0"	},
	{	"i",	"",	"\0"	},
	{	"u",	"",	"\0"	},
	{	"e",	"",	"\0"	},
	{	"o",	"",	"\0"	},

	{	"xa",	"",	"\0"	},
	{	"xi",	"",	"\0"	},
	{	"xu",	"",	"\0"	},
	{	"xe",	"",	"\0"	},
	{	"xo",	"",	"\0"	},

	{	"ka",	"",	"\0"	},
	{	"ki",	"",	"\0"	},
	{	"ku",	"",	"\0"	},
	{	"ke",	"",	"\0"	},
	{	"ko",	"",	"\0"	},

	{	"kya",	"",	"\0"	},
	{	"kyi",	"",	"\0"	},
	{	"kyu",	"",	"\0"	},
	{	"kye",	"",	"\0"	},
	{	"kyo",	"",	"\0"	},

	{	"kwa",	"",	"\0"	},
	{	"kwi",	"",	"\0"	},
	{	"kwu",	"" ,	"\0"	},
	{	"kwe",	"",	"\0"	},
	{	"kwo",	"",	"\0"	},

	{	"qya",	"",	"\0"	},
	{	"qyi",	"",	"\0"	},
	{	"qyu",	"",	"\0"	},
	{	"qye",	"",	"\0"	},
	{	"qyo",	"",	"\0"	},

	{	"sa",	"",	"\0"	},
	{	"si",	"",	"\0"	},
	{	"su",	"",	"\0"	},
	{	"se",	"",	"\0"	},
	{	"so",	"",	"\0"	},

	{	"ci",	"",	"\0"	},
	{	"ce",	"",	"\0"	},

	{	"sya",	"",	"\0"	},
	{	"syi",	"",	"\0"	},
	{	"syu",	"",	"\0"	},
	{	"sye",	"",	"\0"	},
	{	"syo",	"",	"\0"	},

	{	"sha",	"",	"\0"	},
	{	"shi",	"" ,	"\0"	},
	{	"shu",	"",	"\0"	},
	{	"she",	"",	"\0"	},
	{	"sho",	"",	"\0"	},

	{	"ta",	"",	"\0"	},
	{	"ti",	"",	"\0"	},
	{	"tu",	"",	"\0"	},
	{	"te",	"",	"\0"	},
	{	"to",	"",	"\0"	},

	{	"ltu",	"",	"\0"	},
	{	"ltsu",	"",	"\0"	},

	{	"tya",	"",	"\0"	},
	{	"tyi",	"",	"\0"	},
	{	"tyu",	"",	"\0"	},
	{	"tye",	"",	"\0"	},
	{	"tyo",	"",	"\0"	},

	{	"cya",	"",	"\0"	},
	{	"cyi",	"",	"\0"	},
	{	"cyu",	"",	"\0"	},
	{	"cye",	"",	"\0"	},
	{	"cyo",	"",	"\0"	},

	{	"cha",	"",	"\0"	},
	{	"chi",	"" ,	"\0"	},
	{	"chu",	"",	"\0"	},
	{	"che",	"",	"\0"	},
	{	"cho",	"",	"\0"	},

	{	"tsa",	"",	"\0"	},
	{	"tsi",	"",	"\0"	},
	{	"tsu",	"" ,	"\0"	},
	{	"tse",	"",	"\0"	},
	{	"tso",	"",	"\0"	},

	{	"tha",	"Î",	"\0"	},
	{	"thi",	"Î",	"\0"	},
	{	"thu",	"Î",	"\0"	},
	{	"the",	"Î",	"\0"	},
	{	"tho",	"Î",	"\0"	},

	{	"twu",	"Ď",	"\0"	},

	{	"na",	"",	"\0"	},
	{	"ni",	"",	"\0"	},
	{	"nu",	"",	"\0"	},
	{	"ne",	"",	"\0"	},
	{	"no",	"",	"\0"	},

	{	"nya",	"Ǝ",	"\0"	},
	{	"nyi",	"Ǝ",	"\0"	},
	{	"nyu",	"Ǝ",	"\0"	},
	{	"nye",	"Ǝ",	"\0"	},
	{	"nyo",	"Ǝ",	"\0"	},

	{	"ha",	"",	"\0"	},
	{	"hi",	"",	"\0"	},
	{	"hu",	"",	"\0"	},
	{	"he",	"",	"\0"	},
	{	"ho",	"",	"\0"	},

	{	"hya",	"ˎ",	"\0"	},
	{	"hyi",	"ˎ",	"\0"	},
	{	"hyu",	"ˎ",	"\0"	},
	{	"hye",	"ˎ",	"\0"	},
	{	"hyo",	"ˎ",	"\0"	},

	{	"fa",	"̎",	"\0"	},
	{	"fi",	"̎",	"\0"	},
	{	"fu",	"" ,	"\0"	},
	{	"fe",	"̎",	"\0"	},
	{	"fo",	"̎",	"\0"	},

	{	"fya",	"̎",	"\0"	},
	{	"fyi",	"̎",	"\0"	},
	{	"fyu",	"̎",	"\0"	},
	{	"fye",	"̎",	"\0"	},
	{	"fyo",	"̎",	"\0"	},

	{	"ma",	"",	"\0"	},
	{	"mi",	"",	"\0"	},
	{	"mu",	"",	"\0"	},
	{	"me",	"",	"\0"	},
	{	"mo",	"",	"\0"	},

	{	"mya",	"Ў",	"\0"	},
	{	"myi",	"Ў",	"\0"	},
	{	"myu",	"Ў",	"\0"	},
	{	"mye",	"Ў",	"\0"	},
	{	"myo",	"Ў",	"\0"	},

	{	"ya",	"",	"\0"	},
	{	"yi",	"",	"\0"	},
	{	"yu",	"",	"\0"	},
	{	"ye",	"",	"\0"	},
	{	"yo",	"",	"\0"	},

	{	"lya",	"؎",	"\0"	},
	{	"lye",	"؎",	"\0"	},
	{	"lyi",	"؎",	"\0"	},
	{	"lyo",	"؎",	"\0"	},
	{	"lyu",	"؎",	"\0"	},

	{	"xya",	"",	"\0"	},
	{	"xyi",	"",	"\0"	},
	{	"xyu",	"",	"\0"	},
	{	"xye",	"",	"\0"	},
	{	"xyo",	"",	"\0"	},

	{	"ra",	"",	"\0"	},
	{	"ri",	"",	"\0"	},
	{	"ru",	"",	"\0"	},
	{	"re",	"",	"\0"	},
	{	"ro",	"",	"\0"	},

	{	"la",	"",	"\0"	},
	{	"li",	"",	"\0"	},
	{	"lu",	"",	"\0"	},
	{	"le",	"",	"\0"	},
	{	"lo",	"",	"\0"	},

	{	"rya",	"؎",	"\0"	},
	{	"ryi",	"؎",	"\0"	},
	{	"ryu",	"؎",	"\0"	},
	{	"rye",	"؎",	"\0"	},
	{	"ryo",	"؎",	"\0"	},

	{	"wa",	"",	"\0"	},
	{	"wi",	"",	"\0"	},
	{	"wu",	"",	"\0"	},
	{	"we",	"",	"\0"	},
	{	"wo",	"",	"\0"	},

	{	"wha",	"",	"\0"	},
	{	"whi",	"",	"\0"	},
	{	"whu",	"" ,	"\0"	},
	{	"whe",	"",	"\0"	},
	{	"who",	"",	"\0"	},

	{	"xwa",	"",	"\0"	},
	{	"xtu",	"",	"\0"	},
	{	"xtsu",	"",	"\0"	},

	{	"ll",	"",	"l"		},

	{	"nn",	"",	"\0"	},
	{	"mn",	"",	"\0"	},

	{	"ga",	"",	"\0"	},
	{	"gi",	"",	"\0"	},
	{	"gu",	"",	"\0"	},
	{	"ge",	"",	"\0"	},
	{	"go",	"",	"\0"	},

	{	"gya",	"ގ",	"\0"	},
	{	"gyi",	"ގ",	"\0"	},
	{	"gyu",	"ގ",	"\0"	},
	{	"gye",	"ގ",	"\0"	},
	{	"gyo",	"ގ",	"\0"	},

	{	"gwa",	"ގ",	"\0"	},
	{	"gwi",	"ގ",	"\0"	},
	{	"gwu",	"ގ",	"\0"	},
	{	"gwe",	"ގ",	"\0"	},
	{	"gwo",	"ގ",	"\0"	},

	{	"za",	"",	"\0"	},
	{	"zi",	"",	"\0"	},
	{	"zu",	"",	"\0"	},
	{	"ze",	"",	"\0"	},
	{	"zo",	"",	"\0"	},

	{	"jya",	"ގ",	"\0"	},
	{	"jyi",	"ގ",	"\0"	},
	{	"jyu",	"ގ",	"\0"	},
	{	"jye",	"ގ",	"\0"	},
	{	"jyo",	"ގ",	"\0"	},

	{	"zya",	"ގ",	"\0"	},
	{	"zyi",	"ގ",	"\0"	},
	{	"zyu",	"ގ",	"\0"	},
	{	"zye",	"ގ",	"\0"	},
	{	"zyo",	"ގ",	"\0"	},

	{	"ja",	"ގ",	"\0"	},
	{	"ji",	"" ,	"\0"	},
	{	"ju",	"ގ",	"\0"	},
	{	"je",	"ގ",	"\0"	},
	{	"jo",	"ގ",	"\0"	},

	{	"da",	"",	"\0"	},
	{	"di",	"",	"\0"	},
	{	"du",	"",	"\0"	},
	{	"de",	"Î",	"\0"	},
	{	"do",	"Ď",	"\0"	},

	{	"dya",	"ގ",	"\0"	},
	{	"dyi",	"ގ",	"\0"	},
	{	"dyu",	"ގ",	"\0"	},
	{	"dye",	"ގ",	"\0"	},
	{	"dyo",	"ގ",	"\0"	},

	{	"dha",	"Îގ",	"\0"	},
	{	"dhi",	"Îގ",	"\0"	},
	{	"dhu",	"Îގ",	"\0"	},
	{	"dhe",	"Îގ",	"\0"	},
	{	"dho",	"Îގ",	"\0"	},

	{	"dwu",	"Ďގ",	"\0"	},

	{	"ba",	"ʎ",	"\0"	},
	{	"bi",	"ˎ",	"\0"	},
	{	"bu",	"̎",	"\0"	},
	{	"be",	"͎",	"\0"	},
	{	"bo",	"Ύ",	"\0"	},

	{	"bya",	"ˎގ",	"\0"	},
	{	"byi",	"ˎގ",	"\0"	},
	{	"byu",	"ˎގ",	"\0"	},
	{	"bye",	"ˎގ",	"\0"	},
	{	"byo",	"ˎގ",	"\0"	},

	{	"pa",	"ʎ",	"\0"	},
	{	"pi",	"ˎ",	"\0"	},
	{	"pu",	"̎",	"\0"	},
	{	"pe",	"͎",	"\0"	},
	{	"po",	"Ύ",	"\0"	},

	{	"pya",	"ˎߎ",	"\0"	},
	{	"pyi",	"ˎߎ",	"\0"	},
	{	"pyu",	"ˎߎ",	"\0"	},
	{	"pye",	"ˎߎ",	"\0"	},
	{	"pyo",	"ˎߎ",	"\0"	},

	{	"ca",	"",	"\0"	},
	{	"cu",	"",	"\0"	},
	{	"co",	"",	"\0"	},

	{	"xka",	"",	"\0"	},
	{	"xke",	"",	"\0"	},

	{	"va",	"ގ",	"\0"	},
	{	"vi",	"ގ",	"\0"	},
	{	"vu",	"" ,	"\0"	},
	{	"ve",	"ގ",	"\0"	},
	{	"vo",	"ގ",	"\0"	},

	{	"vya",	"ގ",	"\0"	},
	{	"vyi",	"ގ",	"\0"	},
	{	"vyu",	"ގ",	"\0"	},
	{	"vye",	"ގ",	"\0"	},
	{	"vyo",	"ގ",	"\0"	},

	{	"kk",	"",	"k"	},
	{	"ss",	"",	"s"	},
	{	"tt",	"",	"t"	},
	{	"hh",	"",	"h"	},
	{	"mm",	"",	"m"	},
	{	"yy",	"",	"y"	},
	{	"rr",	"",	"r"	},
	{	"ww",	"",	"w"	},
	{	"gg",	"",	"g"	},
	{	"zz",	"",	"z"	},
	{	"dd",	"",	"d"	},
	{	"bb",	"",	"b"	},
	{	"pp",	"",	"p"	},
	{	"cc",	"",	"c"	},
	{	"ff",	"",	"f"	},
	{	"jj",	"",	"j"	},
	{	"qq",	"",	"q"	},
	{	"vv",	"",	"v"	},
	{	"tch",	"",	"ch"	},

	{	"Z ",	" ",	"\0"	},
	{	"@ ",	" ",	"\0"	},
	{	"ZA",	"A",	"\0"	},
	{	"ZB",	"B",	"\0"	},
	{	"ZC",	"C",	"\0"	},
	{	"ZD",	"D",	"\0"	},
	{	"ZE",	"E",	"\0"	},
	{	"ZF",	"F",	"\0"	},
	{	"ZG",	"G",	"\0"	},
	{	"ZH",	"H",	"\0"	},
	{	"ZI",	"I",	"\0"	},
	{	"ZJ",	"J",	"\0"	},
	{	"ZK",	"K",	"\0"	},
	{	"ZL",	"L",	"\0"	},
	{	"ZM",	"M",	"\0"	},
	{	"ZN",	"N",	"\0"	},
	{	"ZO",	"O",	"\0"	},
	{	"ZP",	"P",	"\0"	},
	{	"ZQ",	"Q",	"\0"	},
	{	"ZR",	"R",	"\0"	},
	{	"ZS",	"S",	"\0"	},
	{	"ZT",	"T",	"\0"	},
	{	"ZU",	"U",	"\0"	},
	{	"ZV",	"V",	"\0"	},
	{	"ZW",	"W",	"\0"	},
	{	"ZX",	"X",	"\0"	},
	{	"ZY",	"Y",	"\0"	},
	{	"ZZ",	"Z",	"\0"	},
	{	"Za",	"a",	"\0"	},
	{	"Zb",	"b",	"\0"	},
	{	"Zc",	"c",	"\0"	},
	{	"Zd",	"d",	"\0"	},
	{	"Ze",	"e",	"\0"	},
	{	"Zf",	"f",	"\0"	},
	{	"Zg",	"g",	"\0"	},
	{	"Zh",	"h",	"\0"	},
	{	"Zi",	"i",	"\0"	},
	{	"Zj",	"j",	"\0"	},
	{	"Zk",	"k",	"\0"	},
	{	"Zl",	"l",	"\0"	},
	{	"Zm",	"m",	"\0"	},
	{	"Zn",	"n",	"\0"	},
	{	"Zo",	"o",	"\0"	},
	{	"Zp",	"p",	"\0"	},
	{	"Zq",	"q",	"\0"	},
	{	"Zr",	"r",	"\0"	},
	{	"Zs",	"s",	"\0"	},
	{	"Zt",	"t",	"\0"	},
	{	"Zu",	"u",	"\0"	},
	{	"Zv",	"v",	"\0"	},
	{	"Zw",	"w",	"\0"	},
	{	"Zx",	"x",	"\0"	},
	{	"Zy",	"y",	"\0"	},
	{	"Zz",	"z",	"\0"	},
	{	"Z0",	"0",	"\0"	},
	{	"Z1",	"1",	"\0"	},
	{	"Z2",	"2",	"\0"	},
	{	"Z3",	"3",	"\0"	},
	{	"Z4",	"4",	"\0"	},
	{	"Z5",	"5",	"\0"	},
	{	"Z6",	"6",	"\0"	},
	{	"Z7",	"7",	"\0"	},
	{	"Z8",	"8",	"\0"	},
	{	"Z9",	"9",	"\0"	},
	{	"Z!",	"!",	"\0"	},
	{	"Z\"",	"\"",	"\0"	},
	{	"Z#",	"#",	"\0"	},
	{	"Z$",	"$",	"\0"	},
	{	"Z%",	"%",	"\0"	},
	{	"Z&",	"&",	"\0"	},
	{	"Z'",	"'",	"\0"	},
	{	"Z(",	"(",	"\0"	},
	{	"Z)",	")",	"\0"	},
	{	"Z~",	"~",	"\0"	},
	{	"Z-",	"-",	"\0"	},
	{	"Z=",	"=",	"\0"	},
	{	"Z^",	"^",	"\0"	},
	{	"Z|",	"|",	"\0"	},
	{	"Z@",	"@",	"\0"	},
	{	"Z`",	"`",	"\0"	},
	{	"Z[",	"[",	"\0"	},
	{	"Z{",	"{",	"\0"	},
	{	"Z;",	";",	"\0"	},
	{	"Z+",	"+",	"\0"	},
	{	"Z:",	":",	"\0"	},
	{	"Z*",	"*",	"\0"	},
	{	"Z]",	"]",	"\0"	},
	{	"Z}",	"}",	"\0"	},
	{	"Z<",	"<",	"\0"	},
	{	"Z,",	",",	"\0"	},
	{	"Z>",	">",	"\0"	},
	{	"Z.",	".",	"\0"	},
	{	"Z/",	"/",	"\0"	},
	{	"Z?",	"?",	"\0"	},
	{	"Z\\",	"\\",	"\0"	},
	{	"Z_",	"_",	"\0"	},

	{	",",	",",	"\0"	},
	{	".",	".",	"\0"	},
	{	"[",	"[",	"\0"	},
	{	"]",	"]",	"\0"	},
	{	"{",	"{",	"\0"	},
	{	"}",	"}",	"\0"	},
	{	"^",	"^",	"\0"	},
	{	"<",	"<",	"\0"	},
	{	">",	">",	"\0"	},
	{	"-",	"-",	"\0"	},
	{	"@-",	"-",	"\0"	},
	{	"@@",	"@",	"\0"	},
	{	"@/",	"/",	"\0"	},
	{	"@\\",	"\\",	"\0"	},
	{	"@",	"@",	"\0"	},
	{	"@,",	",",	"\0"	},
	{	"@.",	".",	"\0"	},
	{	"@~",	"~",	"\0"	},
	{	"@||",	"|",	"\0"	},
	{	"@|",	"|",	"\0"	},
	{	"@3",	"3",	"\0"	},
	{	"@2",	"2",	"\0"	},
	{	"@(",	"(",	"\0"	},
	{	"@)",	")",	"\0"	},
	{	"@[",	"[",	"\0"	},
	{	"@]",	"]",	"\0"	},
	{	"@{",	"{",	"\0"	},
	{	"@}",	"}",	"\0"	},

	{	"~",	"~",	"\0"	},
	{	"?",	"?",	"\0"	},
	{	"!",	"!",	"\0"	},
	{	"#",	"#",	"\0"	},
	{	"$",	"$",	"\0"	},
	{	"%",	"%",	"\0"	},
	{	"&",	"&",	"\0"	},
	{	"`",	"`",	"\0"	},
	{	"+",	"+",	"\0"	},
	{	"*",	"*",	"\0"	},
	{	"=",	"=",	"\0"	},
	{	"|",	"|",	"\0"	},
	{	";",	";",	"\0"	},
	{	":",	":",	"\0"	},
	{	"_",	"_",	"\0"	},
	{	"/",	"/",	"\0"	},
	{	"\\",	"\\",	"\0"	},

	{	NULL,	NULL,	NULL	},
};


static const struct rk_map_source_t rk_map_kana_hiragana[] = {
	{	"1",	"",	"\0"	},
	{	"2",	"",	"\0"	},
	{	"3",	"",	"\0"	},
	{	"4",	"",	"\0"	},
	{	"5",	"",	"\0"	},
	{	"6",	"",	"\0"	},
	{	"7",	"",	"\0"	},
	{	"8",	"",	"\0"	},
	{	"9",	"",	"\0"	},
	{	"0",	"",	"\0"	},
	{	"-",	"",	"\0"	},
	{	"^",	"",	"\0"	},
	{	"|",	"",	"\0"	},

	{	"q",	"",	"\0"	},
	{	"w",	"",	"\0"	},
	{	"e",	"",	"\0"	},
	{	"r",	"",	"\0"	},
	{	"t",	"",	"\0"	},
	{	"y",	"",	"\0"	},
	{	"u",	"",	"\0"	},
	{	"i",	"",	"\0"	},
	{	"o",	"",	"\0"	},
	{	"p",	"",	"\0"	},
	{	"@",	"",	"\0"	},
	{	"[",	"",	"\0"	},

	{	"a",	"",	"\0"	},
	{	"s",	"",	"\0"	},
	{	"d",	"",	"\0"	},
	{	"f",	"",	"\0"	},
	{	"g",	"",	"\0"	},
	{	"h",	"",	"\0"	},
	{	"j",	"",	"\0"	},
	{	"k",	"",	"\0"	},
	{	"l",	"",	"\0"	},
	{	";",	"",	"\0"	},
	{	":",	"",	"\0"	},
	{	"]",	"",	"\0"	},

	{	"z",	"",	"\0"	},
	{	"x",	"",	"\0"	},
	{	"c",	"",	"\0"	},
	{	"v",	"",	"\0"	},
	{	"b",	"",	"\0"	},
	{	"n",	"",	"\0"	},
	{	"m",	"",	"\0"	},
	{	",",	"",	"\0"	},
	{	".",	"",	"\0"	},
	{	"/",	"",	"\0"	},
	{	"\\",	"",	"\0"	},
	{	"_",	"",	"\0"	},

	{	"#",	"",	"\0"	},
	{	"E",	"",	"\0"	},
	{	"$",	"",	"\0"	},
	{	"%",	"",	"\0"	},
	{	"&",	"",	"\0"	},
	{	"'",	"",	"\0"	},
	{	"(",	"",	"\0"	},
	{	")",	"",	"\0"	},
	{	"~",	"",	"\0"	},

	{	"{",	"",	"\0"	},
	{	"}",	"",	"\0"	},
	{	"Z",	"",	"\0"	},
	{	"<",	"",	"\0"	},
	{	">",	"",	"\0"	},
	{	"?",	"",	"\0"	},

	{	"2@",	"",	"\0"	},
	{	"4@",	"",	"\0"	},
	{	"-@",	"",	"\0"	},
	{	"^@",	"",	"\0"	},

	{	"q@",	"",	"\0"	},
	{	"w@",	"",	"\0"	},
	{	"r@",	"",	"\0"	},
	{	"t@",	"",	"\0"	},
	{	"p@",	"",	"\0"	},

	{	"a@",	"",	"\0"	},
	{	"s@",	"",	"\0"	},
	{	"d@",	"",	"\0"	},
	{	"f@",	"",	"\0"	},
	{	"g@",	"",	"\0"	},
	{	"h@",	"",	"\0"	},
	{	":@",	"",	"\0"	},

	{	"z@",	"",	"\0"	},
	{	"x@",	"",	"\0"	},
	{	"c@",	"",	"\0"	},
	{	"v@",	"",	"\0"	},
	{	"b@",	"",	"\0"	},

	{	"f[",	"",	"\0"	},
	{	"v[",	"",	"\0"	},
	{	"2[",	"",	"\0"	},
	{	"^[",	"",	"\0"	},
	{	"-[",	"",	"\0"	},

	{	NULL,	NULL,	NULL	},
};


static const struct rk_map_source_t rk_map_kana_katakana[] = {
	{	"1",	"",	"\0"	},
	{	"2",	"",	"\0"	},
	{	"3",	"",	"\0"	},
	{	"4",	"",	"\0"	},
	{	"5",	"",	"\0"	},
	{	"6",	"",	"\0"	},
	{	"7",	"",	"\0"	},
	{	"8",	"",	"\0"	},
	{	"9",	"",	"\0"	},
	{	"0",	"",	"\0"	},
	{	"-",	"",	"\0"	},
	{	"^",	"",	"\0"	},
	{	"|",	"",	"\0"	},

	{	"q",	"",	"\0"	},
	{	"w",	"",	"\0"	},
	{	"e",	"",	"\0"	},
	{	"r",	"",	"\0"	},
	{	"t",	"",	"\0"	},
	{	"y",	"",	"\0"	},
	{	"u",	"",	"\0"	},
	{	"i",	"",	"\0"	},
	{	"o",	"",	"\0"	},
	{	"p",	"",	"\0"	},
	{	"@",	"",	"\0"	},
	{	"[",	"",	"\0"	},

	{	"a",	"",	"\0"	},
	{	"s",	"",	"\0"	},
	{	"d",	"",	"\0"	},
	{	"f",	"",	"\0"	},
	{	"g",	"",	"\0"	},
	{	"h",	"",	"\0"	},
	{	"j",	"",	"\0"	},
	{	"k",	"",	"\0"	},
	{	"l",	"",	"\0"	},
	{	";",	"",	"\0"	},
	{	":",	"",	"\0"	},
	{	"]",	"",	"\0"	},

	{	"z",	"",	"\0"	},
	{	"x",	"",	"\0"	},
	{	"c",	"",	"\0"	},
	{	"v",	"",	"\0"	},
	{	"b",	"",	"\0"	},
	{	"n",	"",	"\0"	},
	{	"m",	"",	"\0"	},
	{	",",	"",	"\0"	},
	{	".",	"",	"\0"	},
	{	"/",	"",	"\0"	},
	{	"\\",	"",	"\0"	},
	{	"_",	"",	"\0"	},

	{	"#",	"",	"\0"	},
	{	"E",	"",	"\0"	},
	{	"$",	"",	"\0"	},
	{	"%",	"",	"\0"	},
	{	"&",	"",	"\0"	},
	{	"'",	"",	"\0"	},
	{	"(",	"",	"\0"	},
	{	")",	"",	"\0"	},
	{	"~",	"",	"\0"	},

	{	"{",	"",	"\0"	},
	{	"}",	"",	"\0"	},
	{	"Z",	"",	"\0"	},
	{	"<",	"",	"\0"	},
	{	">",	"",	"\0"	},
	{	"?",	"",	"\0"	},

	{	"2@",	"",	"\0"	},
	{	"4@",	"",	"\0"	},
	{	"-@",	"",	"\0"	},
	{	"^@",	"",	"\0"	},

	{	"q@",	"",	"\0"	},
	{	"w@",	"",	"\0"	},
	{	"r@",	"",	"\0"	},
	{	"t@",	"",	"\0"	},
	{	"p@",	"",	"\0"	},

	{	"a@",	"",	"\0"	},
	{	"s@",	"",	"\0"	},
	{	"d@",	"",	"\0"	},
	{	"f@",	"",	"\0"	},
	{	"g@",	"",	"\0"	},
	{	"h@",	"",	"\0"	},
	{	":@",	"",	"\0"	},

	{	"z@",	"",	"\0"	},
	{	"x@",	"",	"\0"	},
	{	"c@",	"",	"\0"	},
	{	"v@",	"",	"\0"	},
	{	"b@",	"",	"\0"	},

	{	"f[",	"",	"\0"	},
	{	"v[",	"",	"\0"	},
	{	"2[",	"",	"\0"	},
	{	"^[",	"",	"\0"	},
	{	"-[",	"",	"\0"	},

	{	NULL,	NULL,	NULL	},
};


struct rk_map_tables_t rk_map_tables[] = {
	{	2,		"hiragana",			rk_map_hiragana,		NULL	},
	{	3,		"katakana",			rk_map_katakana,		NULL	},
	{	-1,		"alphabet",			rk_map_alphabet,		NULL	},
	{	-1,		"walphabet",		rk_map_walphabet,		NULL	},
	{	5,		"hankaku_kana",		rk_map_hankaku_kana,	NULL	},
	{	6,		"kana_hiragana",	rk_map_kana_hiragana,	NULL	},
	{	7,		"kana_katakana",	rk_map_kana_katakana,	NULL	},
	{	-1,		NULL,				NULL,					NULL	},
};

/* [ End of File ] */
